----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    12:12:18 06/28/2006 
-- Design Name: 
-- Module Name:    ATM_db - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE,mylib;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use mylib.atm_pack.all;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity ATM_db is
    Port ( clock_p : in  STD_LOGIC;
           clock_n : in  STD_LOGIC;
           clk25m : in  STD_LOGIC;
-- TKO pins
           FN : out  STD_LOGIC_VECTOR (3 downto 0);
           SA : out  STD_LOGIC_VECTOR (10 downto 0);
           D : inout  STD_LOGIC_VECTOR (15 downto 0);
           RSV : in  STD_LOGIC_VECTOR (1 downto 0);
           Q : in  STD_LOGIC;
           DOIT : out  STD_LOGIC;
           YSSIR : in  STD_LOGIC;
           SDSREQ : in  STD_LOGIC;
           G_TRIG : in  STD_LOGIC;
           reset_ATM : out  STD_LOGIC;
-- DDR pins
           MDQ : inout  STD_LOGIC_VECTOR (3 downto 0);
           MA : out  STD_LOGIC_VECTOR (12 downto 0);
           MBA : out  STD_LOGIC_VECTOR (1 downto 0);
           DM : out  STD_LOGIC;
           DQS : inout  STD_LOGIC;
           WE_n : out  STD_LOGIC;
           CAS_n : out  STD_LOGIC;
           RAS_n : out  STD_LOGIC;
           CS_n : out  STD_LOGIC;
           mclke : out  STD_LOGIC;
           mclk : out  STD_LOGIC;
           mclk_n : out  STD_LOGIC;
-- ethernet pins
           reset_b : out  STD_LOGIC;
           TXD : out  STD_LOGIC_VECTOR (3 downto 0);
           TX_clk : in  STD_LOGIC;
           TX_en : out  STD_LOGIC;
           TX_er : out  STD_LOGIC;
           TX_slew : out  STD_LOGIC_VECTOR (1 downto 0);
           RXD : in  STD_LOGIC_VECTOR (3 downto 0);
           RX_clk : in  STD_LOGIC;
           RX_DV : in  STD_LOGIC;
           RX_ER : in  STD_LOGIC;
           COL : in  STD_LOGIC;
           CRS : in  STD_LOGIC;
           PAUSE : out  STD_LOGIC;
           MDIO : inout  STD_LOGIC;
           MDC : out  STD_LOGIC;
           MDINT : in  STD_LOGIC;
           LED : out  STD_LOGIC_VECTOR (2 downto 1);
           CFG : out  STD_LOGIC_VECTOR (3 downto 1);
-- MAC address device
           SSN : inout  STD_LOGIC;
-- IP address switches
           HU : in  STD_LOGIC_VECTOR (2 downto 0);
           CR : in  STD_LOGIC_VECTOR (3 downto 0);
           SL : in  STD_LOGIC_VECTOR (4 downto 0);
-- ATM board SPI control
           SPI_SO : in  STD_LOGIC;
           SPI_SI : out  STD_LOGIC;
           SPI_SCK : out  STD_LOGIC;
           SPI_CSn : out  STD_LOGIC;
-- FLASH interface pins
-- INIT_n high for two clk25m cycles enables one FLASH SPI clock
-- it also enables FLASH chip select
           INIT_n : out  STD_LOGIC;
-- M_Q is FLASH SPI Dout
           M_Q : in  STD_LOGIC;
-- f_ctrl(0) is FLASH SPI Din
-- f_ctrl(1) not used
-- f_ctrl(2) low starts FPGA reprogramming
-- f_ctrl(3) low write protects FLASH
           f_ctrl : inout  STD_LOGIC_VECTOR (3 downto 0));
end ATM_db;

architecture Behavioral of ATM_db is
constant atm_db_ver : std_logic_vector(15 downto 0) := x"0040";
signal clk60 : std_logic; -- 60MHz clock
signal TKO_clk0_dcm : std_logic; -- 60MHz clock
signal TKO_clk : std_logic; -- 60MHz clock
signal TKO_status : std_logic_vector(7 downto 0);
signal rst_clk60 : std_logic;
signal rst_clk60_q : std_logic;
signal clk60_err : std_logic_vector(7 downto 0);
signal clk : std_logic; -- 25MHz clock
signal reset : std_logic;
signal reset_TKO : std_logic;
signal reset_cntr : std_logic;
signal reset_cntr_q : std_logic;
signal reset_err : std_logic;
signal end_resetp : std_logic;
signal end_reset : std_logic;
signal locked : std_logic_vector(2 downto 0);
signal locked0_n : std_logic;
signal locked1_n : std_logic;
signal not_locked : std_logic;
signal mem_test : std_logic;
signal mem_status : std_logic_vector(15 downto 0);
signal SSN_status : std_logic_vector(15 downto 0);
signal status : std_logic_vector(15 downto 0);
signal rst_clk4x : std_logic;
signal clk_dcm : std_logic; -- 25MHz clock
signal clk2x : std_logic;
signal clk4x : std_logic;
signal clk4x_dcm : std_logic;
signal SSN_done : std_logic;
signal SSN_valid : std_logic;
signal SSN_ROM : std_logic_vector(63 downto 0);
signal SSN_CRC : std_logic_vector(7 downto 0);
signal flash_data : std_logic_vector(7 downto 0);
signal other_data : std_logic_vector(7 downto 0);
signal loc_re : std_logic;
signal loc_we : std_logic;
signal loc_ack : std_logic;
signal reg_ack : std_logic;
signal loc_wd : std_logic_vector(7 downto 0);
signal loc_rd : std_logic_vector(7 downto 0);
signal loc_addr : std_logic_vector(31 downto 0);
signal TCP_vers : std_logic_vector(31 downto 0);
signal my_MAC : std_logic_vector(47 downto 0);
signal my_IP : std_logic_vector(31 downto 0);
signal SiTCP_rstn : std_logic;
signal MAC_mux : std_logic_vector(7 downto 0);
signal MII_MAC_FLOW_ENB : std_logic := '0';
signal KEEP_ON : std_logic := '0';
signal FAST_RETRANS_ON : std_logic := '0';
signal KEEP_INTVL_FILL : std_logic_vector(15 downto 0) := (others => '0');
signal KEEP_INTVL_EMPTY : std_logic_vector(15 downto 0) := (others => '0');
signal TOUT_ESTB : std_logic_vector(15 downto 0) := (others => '0');
signal TOUT_DISCNCT : std_logic_vector(15 downto 0) := (others => '0');
signal TOUT_RETRANS : std_logic_vector(15 downto 0) := (others => '0');
signal MSL : std_logic_vector(15 downto 0) := (others => '0');
signal TX_BUF_TH : std_logic_vector(15 downto 0) := (others => '0');
signal TCP_byte_order : std_logic;
signal TCP_ESTABLISH : std_logic;
signal TCP_CLOSE_REQ : std_logic;
signal TCP_CLOSE_ACK : std_logic;
signal RX_FIFO_FILL : std_logic_vector(15 downto 0);
signal TX_FIFO_AFULL : std_logic;
signal TX_FIFO_AFULL_q : std_logic;
signal TX_FIFO_WR : std_logic;
signal TX_FIFO_DATA : std_logic_vector(7 downto 0);
signal TX_CRC : std_logic_vector(15 downto 0) := (others => '0');
signal TKO_CRC : std_logic_vector(15 downto 0) := (others => '0');
signal ATM_data : std_logic_vector(7 downto 0);
signal TKO_data : std_logic_vector(7 downto 0);
signal csr_ack : std_logic;
signal TKO_ack : std_logic;
signal sel_SPI : std_logic;
signal SDS_debug : std_logic;
signal FLASH_ack : std_logic;
signal en_cntr : std_logic;
signal SDS_seq : std_logic_vector(35 downto 0);
signal SDS_seq_mux : std_logic_vector(7 downto 0);
signal TKO_fifo_data : std_logic_vector(15 downto 0);
signal TKO_fifo_wr : std_logic;
signal TKO_fifo_datap : std_logic_vector(15 downto 0);
signal TKO_fifo_wrp : std_logic;
signal div : std_logic_vector(23 downto 0);
signal rst_crs : std_logic_vector(1 downto 0);
signal loc_ac : std_logic_vector(1 downto 0);
signal rw_buf : std_logic_vector(15 downto 0);
signal next_lfsr : std_logic_vector(15 downto 0);
signal UDP_CMD : std_logic;
signal UDP_CMD_sync : std_logic_vector(1 downto 0);
signal UDP_ACK : std_logic;
signal UDP_ACK_sync : std_logic_vector(1 downto 0);
signal tgl_a : std_logic;
signal busy : std_logic;
signal tx_en_i : std_logic;
signal timeout : std_logic;
signal wd : std_logic_vector(15 downto 0);
signal fifo_afull : std_logic;
signal mon_ack : std_logic;
signal mon_data : std_logic_vector(7 downto 0);
signal mon_in : std_logic_vector(15 downto 0);
signal rst_cntr : std_logic_vector(15 downto 0);
signal latch_cntr : std_logic_vector(15 downto 0);
signal err_h : std_logic;
signal MDIO_cmd : std_logic_vector(7 downto 0);
signal MDIO_data : std_logic_vector(15 downto 0);
signal MDIO_sr : std_logic_vector(32 downto 0);
--signal PHY_reg : std_logic_vector(4 downto 0);
signal rd_PHY : std_logic;
signal wr_PHY : std_logic;
signal full_duplex : std_logic;
signal MDIO_cycle : std_logic;
signal t_MDIO : std_logic;
signal end_MDIO : std_logic;
signal start_MDIO : std_logic;
signal allowed_reg : std_logic;
signal wr_reg0 : std_logic;
signal wr_reg4 : std_logic;
signal ce_reg0 : std_logic;
signal ce_reg4 : std_logic;
signal PHY_cmd : std_logic;
signal div20_q : std_logic;
signal div23_q : std_logic;
signal is_link : std_logic;
signal check_link : std_logic;
signal link_up : std_logic;
signal active : std_logic_vector(1 downto 0);
signal blink : std_logic_vector(1 downto 0);
--signal init_reg0 : std_logic_vector(1 downto 0);
--signal init_reg4 : std_logic_vector(1 downto 0);
attribute IOSTANDARD : string;
attribute IOSTANDARD of i_clock : label is "lvds_25";
begin
TX_slew <= "00";
PAUSE <= '0';
reset_b <= '0' when SSN_done = '0' else 'Z';
CFG <= "111" when SSN_done = '0' else "ZZZ";
TX_en <= tx_en_i;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(link_up = '0' or blink(0) = '1')then
	   LED(1) <= '0';
	 else
	   LED(1) <= '1';
	 end if;
    if(TCP_ESTABLISH = '0' or blink(1) = '1')then
	   LED(2) <= '0';
	 else
	   LED(2) <= '1';
	 end if;
	 div20_q <= div(20);
	 if(div(20) = '1')then
	   blink <= "00";
	 elsif(div20_q = '1')then
	   blink <= active;
	 end if;
	 if(div(20) = '0' and div20_q = '1')then
	   active <= "00";
	 else
	   active(1) <= active(1) or TX_FIFO_WR;
	   active(0) <= active(0) or loc_we or loc_re or TX_FIFO_WR;
	 end if;
  end if;
end process;
i_clk: bufg port map(i => clk_dcm, o => clk);
   DCM_clk : DCM
	generic map(CLKIN_PERIOD => 40.0)
   port map (
      CLK0 => clk_dcm,     -- 0 degree DCM CLK ouptput
      CLK2x => clk2x,     -- DCM CLK2x ouptput
      CLKFB => clk,   -- DCM clock feedback
      CLKIN => clk25m,   -- Clock input (from IBUFG, BUFG or DCM)
      PSCLK => '0',   -- Dynamic phase adjust clock input
      PSEN => '0',     -- Dynamic phase adjust enable input
      PSINCDEC => '0', -- Dynamic phase adjust increment/decrement
      DSSEN => '0',   --
      RST => '0',      -- DCM asynchronous reset input
		LOCKED => locked(0)
   );
locked0_n <= not locked(0);
i_rst_clk4x : SRL16
   generic map (
      INIT => X"ffff")
   port map (
      Q => rst_clk4x,       -- SRL data output
      A0 => '1',     -- Select[0] input
      A1 => '1',     -- Select[1] input
      A2 => '1',     -- Select[2] input
      A3 => '1',     -- Select[3] input
      CLK => clk,   -- Clock input
      D => locked0_n        -- SRL data input
   );
i_clk4x: bufg port map(i => clk4x_dcm, o => clk4x);
   DCM_clk4x : DCM
	generic map(CLK_FEEDBACK => "2X",CLKIN_PERIOD => 20.0)
   port map (
      CLK2x => clk4x_dcm,     -- DCM CLK2x ouptput
      CLKFB => clk4x,   -- DCM clock feedback
      CLKIN => clk2x,   -- Clock input (from IBUFG, BUFG or DCM)
      PSCLK => '0',   -- Dynamic phase adjust clock input
      PSEN => '0',     -- Dynamic phase adjust enable input
      PSINCDEC => '0', -- Dynamic phase adjust increment/decrement
      DSSEN => '0',   --
      RST => rst_clk4x,      -- DCM asynchronous reset input
		LOCKED => locked(1)
   );
locked1_n <= not locked(1);
i_not_locked : SRL16
   generic map (
      INIT => X"1111")
   port map (
      Q => not_locked,       -- SRL data output
      A0 => '1',     -- Select[0] input
      A1 => '1',     -- Select[1] input
      A2 => '1',     -- Select[2] input
      A3 => '1',     -- Select[3] input
      CLK => clk2x,   -- Clock input
      D => locked1_n        -- SRL data input
   );
i_SSN_if: SSN_if PORT MAP(
		clk => clk,
		en_SSN => mem_status(15),
		SSN => SSN,
		SSN_ROM => SSN_ROM,
		SSN_CRC => SSN_CRC,
		SSN_valid => SSN_valid,
		SSN_done => SSN_done
	);
i_flash: flash PORT MAP(
		clk => clk,
		reset => '0',
		sel_SPI => sel_SPI,
		re => loc_re,
		we => loc_we,
		wdata => loc_wd,
		rdata => flash_data,
		addr => loc_addr(15 downto 0),
		ack => FLASH_ack,
		reconf_b => f_ctrl(2),
		MQ => M_Q,
		MD => f_ctrl(0),
		MW_n => f_ctrl(3),
		MCS => INIT_n,
		SPI_SO => SPI_SO,
		SPI_SI => SPI_SI,
		SPI_SCK => SPI_SCK,
		SPI_CSn => SPI_CSn
	);
--my_IP <= x"c0a80" & HU & CR & SL;
my_IP <= x"c0a80" & not HU & not CR & not SL;
my_MAC <= x"005051" & SSN_ROM(29 downto 8) & "00";
i_SiTCP: SiTCP_FOR_SK
	port map(
	CLK => clk					,	-- in	: System clock (>12.5MHz)
	RSTn => SiTCP_rstn		,	-- in	: System reset
	VERSION => TCP_vers		,	-- out	: Version # of SiTCP
	MY_TCP_PORT => my_TCP_port	,	-- in	: TCP data port #[15:0]
	MY_MAC_ADDR	=> my_MAC,	-- in	: MAC address[47:0]
	MY_IP_ADDR => my_IP		,	-- in	: IP address[31:0]
	MY_MSS => my_MSS			,	-- in	: MSS [11:0]
	MII_MAC_FLOW_ENB => MII_MAC_FLOW_ENB, -- in	: Enable of MAC flow control(1:Enable 0:Disable)
	KEEP_ON => KEEP_ON, -- in	: Keep alive-on (1:ON, 0:OFF)
	FAST_RETRANS_ON => FAST_RETRANS_ON, -- in	: Fast retransmit (1:ON, 0:OFF)
	KEEP_INTVL_FILL => KEEP_INTVL_FILL, -- in	: Interval time to sent keep-alive-packets when a tx-buffer is not empty. [15:0] (msec)
	KEEP_INTVL_EMPTY => KEEP_INTVL_EMPTY, -- in	: Interval time to sent keep-alive-packets when a tx-buffer is empty. [15:0] (msec)
	TOUT_ESTB => TOUT_ESTB, -- in	: Time-out in establish phase (From recved SYN to receive the next ACK). [15:0] (msec)
	TOUT_DISCNCT => TOUT_DISCNCT, -- in	: Disconnect time-out when a connection is established. [15:0] (msec)
	TOUT_RETRANS => TOUT_RETRANS, -- in	: Time-out in retransmit. [15:0] (microsec)
	MSL => MSL					, -- in	: Min. interval time to allow to connect the next connection. [15:0] (msec)	
	TX_BUF_TH => TX_BUF_TH, -- in	: Transmit buffer threshold. [15:0]
	MII_TX_CLK => tx_clk		,	-- in	: Tx clock(2.5 or 25MHz)
	MII_TX_EN => tx_en_i		,	-- out	: Tx enable
	MII_TXD => txd				,	-- out	: Tx data[3:0]
	MII_TX_ER => tx_er		,	-- out	: TX error
	MII_RX_CLK => rx_clk		,	-- in	: Rx clock(2.5 or 25MHz)
	MII_RX_DV => rx_dv		,	-- in	: Rx data valid
	MII_RXD => rxd				,	-- in	: Rx data[3:0]
	MII_RX_ER => rx_er		,	-- in	: Rx error
	MII_CRS => CRS				,	-- in	: Carrier sense
	MII_COL => COL				,	-- in	: Collision detected
--	MII_MDC => MDC				,	-- out	: Clock for MDIO
--	MII_MDIO => MDIO			,	-- io	: data
	MY_UDP_PORT => my_UDP_port	,	-- in	: UDP port #, For Remote Bus Controller
	LOC_ADDR	=> loc_addr		,	-- out	: Address[31:0]
	LOC_WE => loc_we			,	-- out	: Write enable
	LOC_WD => loc_wd		,	-- out	: Data[7:0]
	LOC_RE => loc_re			,	-- out	: Read enable
	LOC_ACK => loc_ack		,	-- in	: Access acknowledge
	LOC_RD => loc_rd		,	-- in	: Read data[7:0]
	TCP_ESTABLISH => TCP_ESTABLISH	,	-- out	: Connection close
	TCP_CLOSE_REQ => TCP_CLOSE_REQ	, -- out	: Connection close
	TCP_CLOSE_ACK => TCP_CLOSE_ACK	,	-- in	: Acknowledge for close
	RX_FIFO_FILL => RX_FIFO_FILL		,	-- in	: Fill level[15:0]
--	RX_FIFO_WR => RX_FIFO_WR			,	-- out	: Write enable
--	RX_FIFO_DATA => RX_FIFO_DATA		,	-- out	: Write data[7:0]
	TX_FIFO_AFULL => TX_FIFO_AFULL	,	-- out	: Almost full flag
	TX_FIFO_WR => TX_FIFO_WR			,	-- in	: Write enable
	TX_FIFO_DATA => TX_FIFO_DATA			-- in	: Write data[7:0]
);
RX_FIFO_FILL <= x"0000";
loc_rd <= ATM_data when loc_addr(15) = '1' else flash_data
  when loc_addr(10) = '1' else mon_data
  when loc_addr(9) = '1' else MAC_mux
  when loc_addr(6) = '1' else SDS_seq_mux
  when loc_addr(5) = '1' else other_data;
process(loc_addr,FAST_RETRANS_ON,KEEP_ON,MII_MAC_FLOW_ENB,KEEP_INTVL_FILL,KEEP_INTVL_EMPTY,TOUT_ESTB,TOUT_DISCNCT,MSL)
begin
  case loc_addr(4 downto 0) is
    when "00001" => MAC_mux <= "00000" & FAST_RETRANS_ON & KEEP_ON & MII_MAC_FLOW_ENB;
    when "00010" => MAC_mux <= KEEP_INTVL_FILL(15 downto 8);
    when "00011" => MAC_mux <= KEEP_INTVL_FILL(7 downto 0);
    when "00100" => MAC_mux <= KEEP_INTVL_EMPTY(15 downto 8);
    when "00101" => MAC_mux <= KEEP_INTVL_EMPTY(7 downto 0);
    when "00110" => MAC_mux <= TOUT_ESTB(15 downto 8);
    when "00111" => MAC_mux <= TOUT_ESTB(7 downto 0);
    when "01000" => MAC_mux <= TOUT_DISCNCT(15 downto 8);
    when "01001" => MAC_mux <= TOUT_DISCNCT(7 downto 0);
    when "01010" => MAC_mux <= MSL(15 downto 8);
    when "01011" => MAC_mux <= MSL(7 downto 0);
    when "01100" => MAC_mux <= MDIO_data(15 downto 8);
    when "01101" => MAC_mux <= MDIO_data(7 downto 0);
    when "01110" => MAC_mux <= x"00";
    when "01111" => MAC_mux <= MDIO_cmd;
    when "10000" => MAC_mux <= TOUT_RETRANS(15 downto 8);
    when "10001" => MAC_mux <= TOUT_RETRANS(7 downto 0);
    when "10010" => MAC_mux <= TX_BUF_TH(15 downto 8);
    when "10011" => MAC_mux <= TX_BUF_TH(7 downto 0);
    when others => MAC_mux <= x"00";
  end case;
end process;
process(loc_addr,SDS_seq,MDIO_data)
begin
  case loc_addr(3 downto 0) is
    when x"1" => SDS_seq_mux <= x"0" & SDS_seq(35 downto 32);
    when x"2" => SDS_seq_mux <= SDS_seq(31 downto 24);
    when x"3" => SDS_seq_mux <= SDS_seq(23 downto 16);
    when x"4" => SDS_seq_mux <= SDS_seq(15 downto 8);
    when x"5" => SDS_seq_mux <= SDS_seq(7 downto 0);
    when x"6" => SDS_seq_mux <= TX_CRC(15 downto 8);
    when x"7" => SDS_seq_mux <= TX_CRC(7 downto 0);
    when x"8" => SDS_seq_mux <= TKO_CRC(15 downto 8);
    when x"9" => SDS_seq_mux <= TKO_CRC(7 downto 0);
    when others => SDS_seq_mux <= x"00";
  end case;
end process;
process(loc_addr,rw_buf,status,mem_status,
        TCP_vers,clk60_err,SSN_status,SSN_ROM,TKO_data)
begin
  case loc_addr(4 downto 0) is
    when "01000" => other_data <= rw_buf(15 downto 8);
    when "01001" => other_data <= rw_buf(7 downto 0);
    when "01010" => other_data <= status(15 downto 8);
    when "01011" => other_data <= status(7 downto 0);
    when "01100" => other_data <= mem_status(15 downto 8);
    when "01101" => other_data <= mem_status(7 downto 0);
    when "01110" => other_data <= atm_db_ver(15 downto 8);
    when "01111" => other_data <= atm_db_ver(7 downto 0);
    when "10000" => other_data <= TCP_vers(31 downto 24);
    when "10001" => other_data <= TCP_vers(23 downto 16);
    when "10010" => other_data <= TCP_vers(15 downto 8);
    when "10011" => other_data <= TCP_vers(7 downto 0);
    when "10100" => other_data <= x"00";
    when "10101" => other_data <= clk60_err;
    when "10110" => other_data <= SSN_status(15 downto 8);
    when "10111" => other_data <= SSN_status(7 downto 0);
    when "11000" => other_data <= SSN_ROM(63 downto 56);
    when "11001" => other_data <= SSN_ROM(55 downto 48);
    when "11010" => other_data <= SSN_ROM(47 downto 40);
    when "11011" => other_data <= SSN_ROM(39 downto 32);
    when "11100" => other_data <= SSN_ROM(31 downto 24);
    when "11101" => other_data <= SSN_ROM(23 downto 16);
    when "11110" => other_data <= SSN_ROM(15 downto 8);
    when "11111" => other_data <= SSN_ROM(7 downto 0);
	 when others => other_data <= TKO_data;
  end case;
end process;
status(15) <= TCP_ESTABLISH;
status(14) <= TX_FIFO_AFULL;
status(13) <= TCP_byte_order;
status(12 downto 11) <= "00";
status(10 downto 8) <= not locked;
status(7 downto 6) <= "00";
status(5) <= timeout;
status(4) <= fifo_afull;
status(3) <= sel_SPI;
status(2) <= mem_test;
status(1) <= SDS_debug;
status(0) <= f_ctrl(1);
SSN_status <= SSN_CRC & "000000" & SSN_valid & SSN_done;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(SSN_done = '0')then
      div <= (others => '0');
	 else
      div <= div + 1;
	 end if;
	 if(SSN_done = '0')then
	   SiTCP_rstn <= '0';
	 elsif(div(17) = '1')then
	   SiTCP_rstn <= '1';
	 end if;
	 reset_cntr_q <= reset_cntr;
	 if(loc_addr(15 downto 4) /= x"023" and (loc_we = '1' or loc_re = '1'))then
	   UDP_CMD <= not UDP_CMD;
	 end if;
	 if(loc_addr(15 downto 4) /= x"023" and loc_ack = '1' and reset_cntr_q = '0')then
	   UDP_ACK <= not UDP_ACK;
	 end if;
    loc_ack <= FLASH_ack or TKO_ack or reg_ack or wd(15) or csr_ack or mon_ack or end_reset;
	 if(loc_addr(15 downto 8) = reg_file(15 downto 8))then
	   reg_ack <= loc_re or loc_we;
	 else
	   reg_ack <= '0';
	 end if;
	 if(loc_addr(15 downto 0) = csr_addr and loc_we = '1')then
	   reset <= loc_wd(0);
	   reset_TKO <= loc_wd(1) or loc_wd(0);
	   reset_err <= loc_wd(2) or loc_wd(1) or loc_wd(0);
	   reset_cntr <= loc_wd(3) or loc_wd(0);
	   reset_ATM <= loc_wd(7);
	 elsif(end_reset = '1')then
	   reset <= '0';
	   reset_TKO <= '0';
	   reset_err <= '0';
	   reset_cntr <= '0';
	   reset_ATM <= '0';
	 end if;
	 if(loc_addr(15 downto 0) = csr_addr and loc_we = '1')then
	   end_resetp <= '1';
	 else
	   end_resetp <= '0';
	 end if;
	 if(loc_addr(15 downto 1) = csr_addr(15 downto 1) and loc_addr(0) = '0' and loc_we = '1')then
	   mem_test <= loc_wd(0);
		SDS_debug <= loc_wd(1);
		sel_SPI <= loc_wd(2);
	 end if;
	 if(loc_addr(15 downto 0) = x"010a" and loc_we = '1')then
	   TCP_byte_order <= loc_wd(5);
	 end if;
	 if(loc_addr(15 downto 1) = csr_addr(15 downto 1) and loc_addr(0) = '0')then
	   csr_ack <= loc_we or loc_re;
	 else
	   csr_ack <= '0';
	 end if;
	 if(loc_addr(15 downto 1) = rw_buf_addr(15 downto 1) and loc_we = '1')then
	   if(loc_addr(0) = '1')then
	     rw_buf(7 downto 0) <= loc_wd;
	   else
	     rw_buf(15 downto 8) <= loc_wd;
		end if;
	 end if;
	 if(reset = '1')then
	   timeout <= '0';
	 elsif(wd(15) = '1')then
	   timeout <= '1';
	 end if;
	 if(reset = '1')then
	   busy <= '0';
	 elsif(loc_we = '1' or loc_re = '1')then
	   busy <= '1';
	 elsif(loc_ack = '1' or wd(15) = '1')then
	   busy <= '0';
	 end if;
	 if(busy = '0' or wd(15) = '1')then
	   wd <= x"0000";
	 else
	   wd <= wd + 1;
	 end if;
  end if;
end process;
i_end_reset: delayN
  generic map(N=> 17) port map(d => end_resetp, ce => '1', c => clk, q => end_reset);
process(TKO_clk)
begin
  if(TKO_clk'event and TKO_clk = '1')then
    UDP_CMD_sync <= UDP_CMD_sync(0) & UDP_CMD;
	 mon_in(6) <= UDP_CMD_sync(1) xor UDP_CMD_sync(0);
    UDP_ACK_sync <= UDP_ACK_sync(0) & UDP_ACK;
	 mon_in(7) <= UDP_ACK_sync(1) xor UDP_ACK_sync(0);
  end if;
end process;
process(clk,SiTCP_rstn)
begin
  if(SiTCP_rstn = '0')then
    MII_MAC_FLOW_ENB <= '1';
    KEEP_ON <= '1';
    FAST_RETRANS_ON <= '1';
	 KEEP_INTVL_FILL <= x"0002";
	 KEEP_INTVL_EMPTY <= x"07D0";
	 TOUT_ESTB <= x"07D0";
	 TOUT_DISCNCT <= x"3A98";
	 TOUT_RETRANS <= x"03e8";
	 MSL <= x"1B58";
	 TX_BUF_TH <= x"3fe0";
  elsif(clk'event and clk = '1')then
	 if(loc_addr = MAC_ctrl_addr and loc_we = '1')then
	   MII_MAC_FLOW_ENB <= loc_wd(0);
	   KEEP_ON <= loc_wd(1);
	   FAST_RETRANS_ON <= loc_wd(2);
	 end if;
	 if(loc_addr(15 downto 1) = KEEP_INTVL_FILL_addr(15 downto 1) and loc_we = '1')then
	   if(loc_addr(0) = '1')then
	     KEEP_INTVL_FILL(7 downto 0) <= loc_wd;
	   else
	     KEEP_INTVL_FILL(15 downto 8) <= loc_wd;
		end if;
	 end if;
	 if(loc_addr(15 downto 1) = KEEP_INTVL_EMPTY_addr(15 downto 1) and loc_we = '1')then
	   if(loc_addr(0) = '1')then
	     KEEP_INTVL_EMPTY(7 downto 0) <= loc_wd;
	   else
	     KEEP_INTVL_EMPTY(15 downto 8) <= loc_wd;
		end if;
	 end if;
	 if(loc_addr(15 downto 1) = TOUT_ESTB_addr(15 downto 1) and loc_we = '1')then
	   if(loc_addr(0) = '1')then
	     TOUT_ESTB(7 downto 0) <= loc_wd;
	   else
	     TOUT_ESTB(15 downto 8) <= loc_wd;
		end if;
	 end if;
	 if(loc_addr(15 downto 1) = TOUT_DISCNCT_addr(15 downto 1) and loc_we = '1')then
	   if(loc_addr(0) = '1')then
	     TOUT_DISCNCT(7 downto 0) <= loc_wd;
	   else
	     TOUT_DISCNCT(15 downto 8) <= loc_wd;
		end if;
	 end if;
	 if(loc_addr(15 downto 1) = MSL_addr(15 downto 1) and loc_we = '1')then
	   if(loc_addr(0) = '1')then
	     MSL(7 downto 0) <= loc_wd;
	   else
	     MSL(15 downto 8) <= loc_wd;
		end if;
	 end if;
	 if(loc_addr(15 downto 1) = TOUT_RETRANS_addr(15 downto 1) and loc_we = '1')then
	   if(loc_addr(0) = '1')then
	     TOUT_RETRANS(7 downto 0) <= loc_wd;
	   else
	     TOUT_RETRANS(15 downto 8) <= loc_wd;
		end if;
	 end if;
	 if(loc_addr(15 downto 1) = TX_BUF_TH_addr(15 downto 1) and loc_we = '1')then
	   if(loc_addr(0) = '1')then
	     TX_BUF_TH(7 downto 0) <= loc_wd;
	   else
	     TX_BUF_TH(15 downto 8) <= loc_wd;
		end if;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
	 en_cntr <= not reset_cntr and (en_cntr or TX_fifo_wr);
	 tx_fifo_afull_q <= tx_fifo_afull;
-- no TX_fifo_wr while TX_fifo_afull false counter
	 mon_in(9) <= en_cntr and not TX_fifo_wr and not TX_fifo_afull_q and TCP_ESTABLISH;
	 latch_cntr(9) <= TX_fifo_wr;
-- TCP_fifo_afull counter
	 mon_in(10) <= en_cntr and TX_fifo_afull;
	 latch_cntr(10) <= TX_fifo_wr;
-- total time counter
	 mon_in(11) <= en_cntr;
	 latch_cntr(11) <= TX_fifo_wr;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset_TKO = '1')then
	   tgl_a <= '0';
      next_lfsr <= x"0000";
    elsif(TX_fifo_wr = '1')then
	   tgl_a <= not tgl_a;
		if(TCP_byte_order = '0')then
		  if(tgl_a = '0')then
		    next_lfsr(15 downto 8) <= tx_fifo_data;
		  else
	       next_lfsr <= next_lfsr(14 downto 8) & tx_fifo_data & not(next_lfsr(15) xor next_lfsr(14) xor next_lfsr(12) xor tx_fifo_data(3));
		  end if;
		else
		  if(tgl_a = '0')then
		    next_lfsr(7 downto 0) <= tx_fifo_data;
		  else
	       next_lfsr <= tx_fifo_data(6 downto 0) & next_lfsr(7 downto 0) & not(tx_fifo_data(7) xor tx_fifo_data(6) xor tx_fifo_data(4) xor next_lfsr(3));
		  end if;
		end if;
	 end if;
	 if(TX_fifo_wr = '1' and tgl_a = '0' and ((TCP_byte_order = '1' and next_lfsr(7 downto 0) /= tx_fifo_data) or
							                        (TCP_byte_order = '0' and next_lfsr(15 downto 8) /= tx_fifo_data)))then
	   err_h <= '1';
	 else
	   err_h <= '0';
	 end if;
-- pseudo-ramdom check error counter
	 if(err_h = '1' or (tgl_a = '1' and (TCP_byte_order = '0' and next_lfsr(7 downto 0) /= tx_fifo_data)) or
							 (tgl_a = '1' and (TCP_byte_order = '1' and next_lfsr(15 downto 8) /= tx_fifo_data)))then
	   mon_in(15) <= '1';
	 else
	   mon_in(15) <= '0';
	 end if;
	 mon_in(13) <= TCP_ESTABLISH and not TX_FIFO_AFULL;
  end if;
end process;
i_TKO_if: TKO_if PORT MAP(
		clk => clk,
		clk60 => TKO_clk,
		reset => reset_TKO,
		rst_err => reset_err,
		fifo_afull => fifo_afull,
		SDSREQ => SDSREQ,
		SDS_debug => SDS_debug,
		FN => FN,
		SA => SA,
		D => D,
		RSV => RSV,
		Q => Q,
		DOIT => DOIT,
		YSSIR => YSSIR,
		G_TRIG => G_TRIG,
		addr => loc_addr,
		wd => loc_wd,
		we => loc_we,
		re => loc_re,
		TKO_ack => TKO_ack,
		mon_out => mon_in(5 downto 1),
		ATM_data => ATM_data,
		TKO_data => TKO_data,
		SDS_seq_o => SDS_seq,
		TKO_fifo_wr => TKO_fifo_wr,
		TKO_fifo_data => TKO_fifo_data
	);
i_sdram_fifo: sdram_fifo PORT MAP(
		reset => reset_TKO,
		clk => clk,
		TKO_clk => TKO_clk,
		clk4x => clk4x,
		mem_status => mem_status,
		not_locked => not_locked,
		mem_test => mem_test,
		TKO_data => TKO_fifo_data,
		TKO_wr => TKO_fifo_wr,
		fifo_afull => fifo_afull,
		TCP_byte_order => TCP_byte_order,
		TCP_ESTABLISH => TCP_ESTABLISH,
		TCP_CLOSE_REQ => TCP_CLOSE_REQ,
		TCP_CLOSE_ACK => TCP_CLOSE_ACK,
		TX_fifo_afull => TX_FIFO_AFULL,
		TX_fifo_we => TX_FIFO_WR,
		TX_fifo_data => TX_FIFO_DATA,
		mdq => mdq,
		maddr => ma,
		mba => mba,
		dm => dm,
		dqs => dqs,
		we_n => we_n,
		ras_n => ras_n,
		cas_n => cas_n,
		cs_n => cs_n,
		mclke => mclke,
		mclk => mclk,
		mclk_n => mclk_n
	);
i_crc16D16: crc16D16 PORT MAP(
		clk => TKO_clk,
		init_crc => reset_TKO,
		we_crc => TKO_fifo_wrp,
		d => TKO_fifo_datap,
		crc => TKO_CRC
	);
process(TKO_clk)
begin
  if(TKO_clk'event and TKO_clk = '1')then
    TKO_fifo_wrp <= TKO_fifo_wr;
	 if(TCP_byte_order = '1')then
	   TKO_fifo_datap <= TKO_fifo_data(7 downto 0) & TKO_fifo_data(15 downto 8);
	 else
	   TKO_fifo_datap <= TKO_fifo_data;
	 end if;
  end if;
end process;
i_crc16D8: crc16D8 PORT MAP(
		clk => clk,
		init_crc => reset_TKO,
		we_crc => TX_FIFO_WR,
		d => TX_FIFO_DATA,
		crc => TX_CRC
	);
mon_in(0) <= mem_status(8);
mon_in(8) <= tgl_a;
-- TCP connection on time counter
mon_in(12) <= TCP_ESTABLISH;
mon_in(14) <= TX_FIFO_wr;
g_cntr: for i in 0 to 7 generate
  rst_cntr(i) <= reset_cntr;
end generate;
rst_cntr(8) <= reset_TKO;
g_cntr2: for i in 9 to 15 generate
  rst_cntr(i) <= reset_cntr;
end generate;
latch_cntr(8 downto 0) <= "111111111";
latch_cntr(15 downto 12) <= "1111";
i_monitor: monitor PORT MAP(
		clk => clk,
		clk4x => clk4x,
		TKO_clk => TKO_clk,
		reset => rst_cntr,
		latch => latch_cntr,
		addr => loc_addr(15 downto 0),
		re => loc_re,
		we => loc_we,
		mon_in => mon_in,
		ack => mon_ack,
		dout => mon_data
	);
i_clock: ibufgds port map(i => clock_p, ib => clock_n, o => clk60);
i_TKO_clk: bufg port map(i => TKO_clk0_dcm, o => TKO_clk);
i_dcm_clk60 : DCM
	generic map(CLKIN_PERIOD => 16.6)
   port map (
      CLK0 => TKO_clk0_dcm,     -- 0 degree DCM CLK ouptput
      CLKFB => TKO_clk,   -- DCM clock feedback
      CLKIN => clk60,   -- Clock input (from IBUFG, BUFG or DCM)
      PSCLK => '0',   -- Dynamic phase adjust clock input
      PSEN => '0',     -- Dynamic phase adjust enable input
      PSINCDEC => '0', -- Dynamic phase adjust increment/decrement
      DSSEN => '0',   --
      RST => rst_clk60,      -- DCM asynchronous reset input
		STATUS => TKO_status,
		LOCKED => locked(2)
   );
i_rst_clk60 : SRL16
   generic map (
      INIT => X"ffff")
   port map (
      Q => rst_clk60,
      A0 => '1',
      A1 => '1',
      A2 => '1',
      A3 => '1',
      CLK => clk,
      D => TKO_status(1)
   );
-- MDIO, MDC interface
MDIO <= 'Z' when t_MDIO = '1' else
--     init_reg0(1) when wr_reg0 = '1' else
--     init_reg4(1) when wr_reg4 = '1' else
	  MDIO_sr(32);
process(clk)
begin
  if(clk'event and clk = '1')then
    MDC <= div(1);
	 rst_clk60_q <= rst_clk60;
	 div23_q <= div(23);
	 if(div23_q = '0' and div(23) = '1')then
	   check_link <= '1';
	 elsif(MDIO_cycle = '1' and is_link = '1')then
	   check_link <= '0';
	 end if;
	 if(reset_err = '1' or SiTCP_rstn = '0')then
	   clk60_err <= (others => '0');
	 elsif(rst_clk60_q = '0' and rst_clk60 = '1' and clk60_err /= x"ff")then
	   clk60_err <= clk60_err + 1;
	 end if;
--	 if(SiTCP_rstn = '0' and div(15 downto 7) = "111111000")then
--	   wr_reg0 <= '1';
--	 else
--	   wr_reg0 <= '0';
--	 end if;
--	 if(SiTCP_rstn = '0' and div(15 downto 7) = "111111100")then
--	   wr_reg4 <= '1';
--	 else
--	   wr_reg4 <= '0';
--	 end if;
--	 ce_reg0 <= wr_reg0 and div(1) and div(0);
--	 ce_reg4 <= wr_reg4 and div(1) and div(0);
	 if(MDIO_cycle = '0')then
	   t_MDIO <= '0';
	 elsif((MDIO_cmd(5) = '0' or is_link = '1') and div(5 downto 0) = "110100")then
	   t_MDIO <= '1';
	 end if;
	 if(MDIO_cycle = '1' and is_link = '1' and div(6 downto 0) = "1010100")then
	   link_up <= MDIO;
	 end if;
	 if(loc_we = '1' and loc_addr(15 downto 1) = MDIO_data_addr(15 downto 1))then
	   if(loc_addr(0) = '1')then
	     MDIO_data(7 downto 0) <= loc_wd;
		else
	     MDIO_data(15 downto 8) <= loc_wd;
		end if;
	 elsif(MDIO_cycle = '1' and MDIO_cmd(5) = '0' and is_link = '0' and div(6) = '1' and div(1 downto 0) = "00")then
	   MDIO_data <= MDIO_data(14 downto 0) & MDIO;
	 end if;
	 if(MDIO_cycle = '0')then
--	   MDIO_sr <= "101" & rd_PHy & wr_PHY & "00000" & PHY_reg & "100010000" & full_duplex;
      if(PHY_cmd = '1')then
	     MDIO_sr <= "101" & not MDIO_cmd(5) & MDIO_cmd(5) & "00000" & MDIO_cmd(4 downto 0) & "10" & MDIO_data;
		else
	     MDIO_sr <= "10110000001000110" & MDIO_data;
		end if;
		is_link <= not PHY_cmd;
	 elsif(div(1 downto 0) = "00")then
	   MDIO_sr <= MDIO_sr(31 downto 0) & '0';
	 end if;
	 if(SSN_done = '0' or end_MDIO = '1')then
	   MDIO_cycle <= '0';
--	 elsif(div(7 downto 0) = x"fb" and (rd_PHY = '1' or wr_PHY = '1'))then
	 elsif(div(7 downto 0) = x"fb" and (PHY_cmd = '1' or check_link = '1'))then
	   MDIO_cycle <= '1';
	 end if;
	 if(MDIO_cycle = '1' and div(7 downto 5) = "101")then
	   end_MDIO <= '1';
	 else
	   end_MDIO <= '0';
	 end if;
	 if(loc_we = '1' and loc_addr(15 downto 1) = MDIO_cmd_addr(15 downto 1) and loc_addr(0) = '1')then
	   MDIO_cmd(5 downto 0) <= loc_wd(5 downto 0);
	   start_MDIO <= '1';
	 else
	   start_MDIO <= '0';		
	 end if;
	 if(MDIO_cycle = '1' and is_link = '0')then
	   PHY_cmd <= '0';
	 elsif(start_MDIO = '1')then
	   PHY_cmd <= allowed_reg;
	 end if;
	 if(SiTCP_rstn = '0')then
		MDIO_cmd(7 downto 6) <= "00";	
	 elsif(start_MDIO = '1')then
		MDIO_cmd(7 downto 6) <= not allowed_reg & '0';	
	 elsif(end_MDIO = '1' and is_link = '0')then
	   MDIO_cmd(6) <= '1';
    end if;	 
--	 if(MDIO_cycle = '1')then
--	   PHY_reg <= "00000";
--	 elsif(loc_we = '1' and loc_addr(15 downto 0) = x"0127")then
--	   PHY_reg <= loc_wd(4 downto 0);
--	 end if;
--	 if(SSN_done = '0' or end_MDIO = '1')then
--	   rd_PHY <= '0';
--	 elsif(loc_we = '1' and loc_addr(15 downto 0) = x"0127")then
--		rd_PHY <= '1';
--	 end if;
--	 if(loc_we = '1' and loc_addr(15 downto 0) = x"0161" and loc_wd = x"5a")then
--	   full_duplex <= not full_duplex;
--	 end if;
--	 if(SSN_done = '0' or end_MDIO = '1')then
--	   wr_PHY <= '0';
--	   rd_PHY <= '0';
--	 elsif(start_MDIO = '1')then
--		wr_PHY <= MDIO_cmd(5);
--		rd_PHY <= not MDIO_cmd(5);
--	 end if;
  end if;
end process;
i_allowed_reg : ROM64X1
   generic map (
      INIT => X"00010091401f01ff")
   port map (
      O => allowed_reg,   -- ROM output
      A0 => MDIO_cmd(0), -- ROM address[0]
      A1 => MDIO_cmd(1), -- ROM address[1]
      A2 => MDIO_cmd(2), -- ROM address[2]
      A3 => MDIO_cmd(3), -- ROM address[3]
      A4 => MDIO_cmd(4), -- ROM address[4]
      A5 => MDIO_cmd(5)  -- ROM address[5]
   );
--i_init_ltx_reg0a : SRL16E
--   generic map (
--      INIT => X"3100")
--   port map (
--      Q => init_reg0(0),
--      A0 => '1',
--      A1 => '1',
--      A2 => '1',
--      A3 => '1',
--      CE => ce_reg0,
--      CLK => clk,
--      D => '1'
--   );
--i_init_ltx_reg0b : SRL16E
--   generic map (
--      INIT => X"5002")
--   port map (
--      Q => init_reg0(1),
--      A0 => '1',
--      A1 => '1',
--      A2 => '1',
--      A3 => '1',
--      CE => ce_reg0,
--      CLK => clk,
--      D => init_reg0(0)
--   );
--i_init_ltx_reg4a : SRL16E
--   generic map (
--      INIT => X"01e1")
--   port map (
--      Q => init_reg4(0),
--      A0 => '1',
--      A1 => '1',
--      A2 => '1',
--      A3 => '1',
--      CE => ce_reg4,
--      CLK => clk,
--      D => '1'
--   );
--i_init_ltx_reg4b : SRL16E
--   generic map (
--      INIT => X"5012")
--   port map (
--      Q => init_reg4(1),
--      A0 => '1',
--      A1 => '1',
--      A2 => '1',
--      A3 => '1',
--      CE => ce_reg4,
--      CLK => clk,
--      D => init_reg4(0)
--   );
end Behavioral;

