/*******************************************************************************
*                                                                              *
* System      : None                                                           *
* Block       : SiTCP                                                          *
* Module      : SiTCP_FOR_SK                                                   *
* Version     : v 0.0.0 2006/07/13 13:06                                       *
*               v 0.1.5 2007/04/23 Added parameter registers                   *
*               v 0.1.6 2007/05/22 Added registers for re-transmission         *
*               v 0.1.7 2007/06/01 Added registers for threshold of the Tx-buffer *
*                                                                              *
* Description :                                                                *
*               SiTCP for SK                                                   *
*                                                                              *
* Designer    : Tomohisa Uchida                                                *
*                                                                              *
*                Copyright (c) 2007 Tomohisa Uchida                            *
*                All rights reserved                                           *
*                                                                              *
*******************************************************************************/
module	SiTCP_FOR_SK(
// System
	CLK					,	// in	: System clock (>12.5MHz)
	RSTn				,	// in	: System reset
// For debug
//	LED_TCP				,	// out	: TCP status[3:0]
	VERSION				,	// out	: Version #
// MAC and IP address
	MY_TCP_PORT			,	// in	: TCP data port #[15:0]
	MY_MAC_ADDR			,	// in	: MAC address[47:0]
	MY_IP_ADDR			,	// in	: IP address[31:0]
	MY_MSS				,	// in	: MSS [11:0]
// Parameters
	MII_MAC_FLOW_ENB	,	// in	: Enable of MAC flow control(1:Enable 0:Disable)
	KEEP_ON				,	// in	: Keep alive-on (1:ON, 0:OFF)
	KEEP_INTVL_FILL		,	// in	: Interval time to sent keep-alive-packets when a tx-buffer is not empty. [15:0] (msec)
	KEEP_INTVL_EMPTY	,	// in	: Interval time to sent keep-alive-packets when a tx-buffer is empty. [15:0] (msec)
	TOUT_ESTB			,	// in	: Time-out in establish phase (From recved SYN to receive the next ACK). [15:0] (msec)
	TOUT_DISCNCT		,	// in	: Disconnect time-out when a connection is established. [15:0] (msec)
	MSL					,	// in	: Min. interval time to allow to connect the next connection. [15:0] (msec)	
	FAST_RETRANS_ON		,	// in	: Fast re-transmission (1:ON, 0:OFF)
	TOUT_RETRANS		,	// in	: Re-transmission time-out[15:0]
	TX_BUF_TH			,	// in	: Threshold of the TX-buffer[15:0]
// MII interface
	// TX
	MII_TX_CLK			,	// in	: Tx clock(2.5 or 25MHz)
	MII_TX_EN			,	// out	: Tx enable
	MII_TXD				,	// out	: Tx data[3:0]
	MII_TX_ER			,	// out	: TX error
	// RX
	MII_RX_CLK			,	// in	: Rx clock(2.5 or 25MHz)
	MII_RX_DV			,	// in	: Rx data valid
	MII_RXD				,	// in	: Rx data[3:0]
	MII_RX_ER			,	// in	: Rx error
	MII_CRS				,	// in	: Carrier sense
	MII_COL				,	// in	: Collision detected
	// Management IF
	MII_MDC				,	// out	: Clock for MDIO
	MII_MDIO			,	// in	: data
// UDP
	MY_UDP_PORT			,	// in	: UDP port #, For Remote Bus Controller
	// Local bus interface
	LOC_ADDR			,	// out	: Address[31:0]
	LOC_WE				,	// out	: Write enable
	LOC_WD				,	// out	: Data[7:0]
	LOC_RE				,	// out	: Read enable
	LOC_ACK				,	// in	: Access acknowledge
	LOC_RD				,	// in	: Read data[7:0]
// TCP connection control
	TCP_ESTABLISH		,	// out	: Connection established
	TCP_CLOSE_REQ		,	// out	: Connection close
	TCP_CLOSE_ACK		,	// in	: Acknowledge for close
// FIFO I/F
	RX_FIFO_FILL		,	// in	: Fill level[15:0]
	RX_FIFO_WR			,	// out	: Write enable
	RX_FIFO_DATA		,	// out	: Write data[7:0]
	TX_FIFO_AFULL		,	// out	: Almost full flag
	TX_FIFO_WR			,	// in	: Write enable
	TX_FIFO_DATA			// in	: Write data[7:0]
);

//-------- Input/Output -------------
	input			CLK				;
	input			RSTn			;

	// LED
//	LED_TCP				,	// out	: TCP status[3:0]

	output	[31:0]	VERSION			;

	input	[15:0]	MY_TCP_PORT		;
	input	[11:0]	MY_MSS			;
	input	[47:0]	MY_MAC_ADDR		;
	input	[31:0]	MY_IP_ADDR		;

	input			MII_MAC_FLOW_ENB;
	input			KEEP_ON			;
	input	[15:0]	KEEP_INTVL_FILL	;
	input	[15:0]	KEEP_INTVL_EMPTY;
	input	[15:0]	TOUT_ESTB		;
	input	[15:0]	TOUT_DISCNCT	;
	input	[15:0]	MSL				;
	input			FAST_RETRANS_ON	;
	input	[15:0]	TOUT_RETRANS	;
	input	[15:0]	TX_BUF_TH		;

	input			MII_TX_CLK		;
	output			MII_TX_EN		;
	output	[3:0]	MII_TXD			;
	output			MII_TX_ER		;
	input			MII_RX_CLK		;
	input			MII_RX_DV		;
	input	[3:0]	MII_RXD			;
	input			MII_RX_ER		;
	input			MII_CRS			;
	input			MII_COL			;
	output			MII_MDC			;
	inout			MII_MDIO		;

	input	[15:0]	MY_UDP_PORT		;
	output	[31:0]	LOC_ADDR		;
	output			LOC_WE			;
	output	[7:0]	LOC_WD			;
	output			LOC_RE			;
	input			LOC_ACK			;
	input	[7:0]	LOC_RD			;

	output			TCP_ESTABLISH	;
	output			TCP_CLOSE_REQ	;
	input			TCP_CLOSE_ACK	;

	input	[15:0]	RX_FIFO_FILL	;
	output			RX_FIFO_WR		;
	output	[7:0]	RX_FIFO_DATA	;
	output			TX_FIFO_AFULL	;
	input			TX_FIFO_WR		;
	input	[7:0]	TX_FIFO_DATA	;

endmodule
