----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    17:03:26 07/20/2006 
-- Design Name: 
-- Module Name:    TKO_if - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE,mylib;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use mylib.atm_pack.all;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity TKO_if is
    Port ( clk : in  STD_LOGIC;
			  clk60 : IN std_logic;
           reset : in  STD_LOGIC;
           rst_err : in  STD_LOGIC;
           fifo_afull : in  STD_LOGIC;
           SDSREQ : in  STD_LOGIC;
-- if SDS_debug set to '1', no flag cells and no data throw-away
           SDS_debug : in  STD_LOGIC;
           FN : out  STD_LOGIC_VECTOR (3 downto 0);
           SA : out  STD_LOGIC_VECTOR (10 downto 0);
           D : inout  STD_LOGIC_VECTOR (15 downto 0);
           RSV : in  STD_LOGIC_VECTOR (1 downto 0);
           Q : in  STD_LOGIC;
           DOIT : out  STD_LOGIC;
           YSSIR : in  STD_LOGIC;
           G_TRIG : in  STD_LOGIC;
           addr : in  STD_LOGIC_VECTOR (31 downto 0);
           wd : in  STD_LOGIC_VECTOR (7 downto 0);
           we : in  STD_LOGIC;
           re : in  STD_LOGIC;
           TKO_ack : out  STD_LOGIC;
           mon_out : out  STD_LOGIC_VECTOR (4 downto 0);
           ATM_data : out  STD_LOGIC_VECTOR (7 downto 0);
           TKO_data : out  STD_LOGIC_VECTOR (7 downto 0);
           SDS_seq_o : out  STD_LOGIC_VECTOR (35 downto 0);
           TKO_fifo_wr : out  STD_LOGIC;
           TKO_fifo_data : out  STD_LOGIC_VECTOR (15 downto 0)
			  );
end TKO_if;

architecture Behavioral of TKO_if is
signal reset_sync : std_logic;
signal reset_TKO : std_logic;
signal fifo_read : std_logic;
signal fifo_read_q : std_logic;
signal t_D : std_logic;
signal DOIT_i : std_logic;
signal UDP_err : std_logic;
signal UDP_TKO : std_logic;
signal UDP_rqst : std_logic;
signal UDP_cycle : std_logic;
signal en_ack : std_logic;
signal div_time : std_logic_vector(13 downto 0) := (others => '0');
signal trig_set : std_logic_vector(15 downto 0) := (others => '1');
signal trig_cntr : std_logic_vector(16 downto 0) := (others => '0');
signal FN_UDP : std_logic_vector(3 downto 0) := x"0";
signal SA_UDP : std_logic_vector(10 downto 0) := "00000000000";
signal FN_i : std_logic_vector(3 downto 0) := x"0";
signal SA_i : std_logic_vector(10 downto 0) := "00000000000";
signal wdata : std_logic_vector(15 downto 0) := x"0000";
signal fifo_data : std_logic_vector(15 downto 0) := x"0000";
signal rdata : std_logic_vector(15 downto 0) := x"0000";
signal tick : std_logic_vector(6 downto 0) := "0000000";
signal en_ack_sync : std_logic_vector(1 downto 0) := "00";
signal got_YSSIR_n : std_logic := '0';
signal got_Q : std_logic := '0';
signal got_YSSIR_UDP : std_logic := '0';
signal got_Q_UDP : std_logic := '0';
signal start_TKO : std_logic := '0';
signal G_trig_sync : std_logic_vector(1 downto 0) := "00";
signal start_fifo_rd : std_logic;
signal stop_fifo_rd : std_logic;
signal start_fifo_rd_sync : std_logic_vector(1 downto 0) := "00";
signal stop_fifo_rd_sync : std_logic_vector(1 downto 0) := "00";
signal UDP_sync : std_logic_vector(1 downto 0) := "00";
signal timer : std_logic_vector(16 downto 0);
signal timer_set : std_logic_vector(15 downto 0);
signal SDS_ctrl : std_logic_vector(7 downto 0);
signal SDS_state : std_logic_vector(2 downto 0);
signal SDS_seq : std_logic_vector(35 downto 0);
signal SDS_cnt : std_logic_vector(32 downto 0);
signal SDS_cnt_ovf : std_logic := '0';
signal SDS_YSSIR_err : std_logic := '0';
signal SDS_cell_err : std_logic := '0';
signal cell_cnt : std_logic_vector(1 downto 0) := "00";
signal cycle_cnt : std_logic_vector(1 downto 0) := "00";
signal partial_flag : std_logic;
signal UDP_stop : std_logic;
signal TKO_status : std_logic_vector(15 downto 0);
signal stop_by_UDP : std_logic;
signal stop_by_Q : std_logic;
signal start_by_UDP_p : std_logic;
signal start_by_timer_p : std_logic;
signal start_by_trig_p : std_logic;
signal start_by_ATM_P : std_logic;
signal start_by_UDP : std_logic;
signal start_by_timer : std_logic;
signal start_by_trig : std_logic;
signal start_by_ATM : std_logic;
attribute init : string;
attribute init of got_YSSIR_n : signal is "S";
attribute init of DOIT_i : signal is "S";
attribute init of t_D : signal is "S";
attribute init of en_ack : signal is "R";
begin
D <= not wdata when t_D = '0' else (others => 'Z');
FN <= not FN_i;
SA <= not SA_i;
DOIT <= DOIT_i;
TKO_fifo_data <= fifo_data;
ATM_data <= rdata(7 downto 0) when addr(0) = '1' else rdata(15 downto 8);
SDS_seq_o <= SDS_seq;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   SDS_ctrl <= x"00";
    elsif(addr = SDS_ctrl_addr and we = '1')then
		SDS_ctrl <= wd;
	 end if;
    if(addr = SDS_ss_addr and we = '1')then
		start_fifo_rd <= wd(2);
		stop_fifo_rd <= wd(1);
	 else	
		start_fifo_rd <= '0';
		stop_fifo_rd <= '0';
	 end if;
    if(reset = '1')then
	   timer_set <= x"0000";
    elsif(addr(15 downto 1) = SDS_timer_addr(15 downto 1) and we = '1')then
	   if(addr(0) = '1')then
		  timer_set(7 downto 0) <= wd;
		else
        timer_set(15 downto 8) <= wd;
		end if;
	 end if;
    if(reset = '1')then
	   trig_set <= x"0000";
    elsif(addr(15 downto 1) = SDS_trig_addr(15 downto 1) and we = '1')then
	   if(addr(0) = '1')then
		  trig_set(7 downto 0) <= wd;
		else
        trig_set(15 downto 8) <= wd;
		end if;
	 end if;
    if((we = '1' or re = '1') and addr(15) = '1')then
	   FN_UDP <= we & addr(14 downto 12);
	   SA_UDP <= addr(11 downto 1);
	 end if;
    if(we = '1' and addr(15) = '1')then
	   if(addr(0) = '0')then
	     wdata(15 downto 8) <= wd;
		else
	     wdata(7 downto 0) <= wd;
		end if;
	 end if;
	 en_ack_sync <= en_ack_sync(0) & en_ack;
	 if(addr(15) = '1' and (we = '1' or (re = '1' and addr(0) = '1') or
		                     en_ack_sync = "10"))then
	   TKO_ack <= '1';
	 else
	   TKO_ack <= '0';
	 end if;
	 if(rst_err = '1')then
	   UDP_err <= '0';
	 elsif(addr(15 downto 12) = x"8" and SDS_ctrl(7 downto 4) /= x"0" and (we = '1' or re = '1'))then
	   UDP_err <= '1';
	 end if;
	 if(addr(15) = '1' and (addr(14 downto 12) /= "000" or SDS_ctrl(7 downto 4) = x"0"))then
	   UDP_TKO <= (we and addr(0)) or (re and not addr(0));
    else
	   UDP_TKO <= '0';
	 end if;
  end if;
end process;
process(addr,trig_set,timer_set,TKO_status,SDS_ctrl)
begin
  case addr(2 downto 0) is
    when "000" => TKO_data <= trig_set(15 downto 8);
    when "001" => TKO_data <= trig_set(7 downto 0);
    when "010" => TKO_data <= timer_set(15 downto 8);
    when "011" => TKO_data <= timer_set(7 downto 0);
    when "100" => TKO_data <= TKO_status(15 downto 8);
    when "101" => TKO_data <= TKO_status(7 downto 0);
    when "111" => TKO_data <= SDS_ctrl;
	 when others => TKO_data <= x"00";
  end case;
end process;
TKO_status(15) <= SDS_YSSIR_err;
TKO_status(14) <= UDP_err;
TKO_status(13) <= SDS_cell_err;
TKO_status(12) <= SDS_cnt_ovf;
TKO_status(11) <= fifo_read;
TKO_status(10) <= fifo_afull;
TKO_status(9) <= got_YSSIR_UDP;
TKO_status(8) <= got_Q_UDP;
TKO_status(7) <= not got_YSSIR_n;
TKO_status(6) <= got_Q;
TKO_status(5) <= start_by_ATM;
TKO_status(4) <= start_by_timer;
TKO_status(3) <= start_by_trig;
TKO_status(2) <= start_by_UDP;
TKO_status(1) <= stop_by_UDP;
TKO_status(0) <= stop_by_Q;
process(clk60)
begin
  if(clk60'event and clk60 = '1')then
    reset_sync <= reset;
    reset_TKO <= reset_sync;
    if(start_TKO = '1')then
	   if(UDP_rqst = '1')then
	     FN_i <= FN_UDP;
	     SA_i <= SA_UDP;
		else
	     FN_i <= x"0";
		  SA_i <= (others => '0');
		end if;
	 end if;
	 case SDS_state is
	   when "001" =>
		  case cycle_cnt is
		    when "00" => fifo_data <= SDS_BOF & SDS_seq(3 downto 0);
		    when "01" => fifo_data <= SDS_seq(19 downto 4);
		    when others => fifo_data <= SDS_seq(35 downto 20);
		  end case;
	   when "010" => fifo_data <= not D;
		when "011" =>
		  if(cycle_cnt = "00")then
		    fifo_data <= SDS_OVF & SDS_seq(3 downto 0);
		  else
		    fifo_data <= x"0000";
		  end if;
		when "101" => fifo_data <= SDS_EOF & SDS_seq(3 downto 0);
		when "110" => fifo_data <= SDS_cnt(31 downto 16);
		when "111" => fifo_data <= SDS_cnt(15 downto 0);
		when others => fifo_data <= x"0000";
	 end case;
    if(en_ack = '1' and tick(5) = '1')then
	   rdata <= not D;
	 end if;
	 if(tick(4 downto 0) = "00000")then
      DOIT_i <= '1';
      t_D <= '1';
	 else
      DOIT_i <= '0';
      t_D <= not FN_i(3);
	 end if;
  end if;
end process;
process(clk60,rst_err)
begin
  if(rst_err = '1')then
    SDS_YSSIR_err <= '0';
    SDS_cell_err <= '0';
	 SDS_cnt_ovf <= '0';
  elsif(clk60'event and clk60 = '1')then
	 if(UDP_cycle = '0' and tick(5) = '1' and got_yssir_n = '1')then
	   SDS_YSSIR_err <= '1';
	 end if;
	 if(UDP_cycle = '0' and tick(5) = '1' and Q = '1' and cell_cnt /= "00")then
	   SDS_cell_err <= '1';
	 end if;
	 SDS_cnt_ovf <= SDS_cnt_ovf or SDS_cnt(32);
  end if;
end process;
process(clk60)
begin
  if(clk60'event and clk60 = '1')then
    if(reset_TKO = '1')then
      en_ack <= '0';
      start_TKO <= '0';
      got_YSSIR_n <= '1';
      got_Q_UDP <= '0';
      got_YSSIR_UDP <= '0';
      got_Q <= '0';
      UDP_sync <= "00";
      UDP_rqst <= '0';
      UDP_cycle <= '0';
      UDP_stop <= '0';
      G_trig_sync <= "00";
      start_fifo_rd_sync <= "00";
      stop_fifo_rd_sync <= "00";
      tick <= (others => '0');
      trig_cntr <= (others => '0');
      div_time <= (others => '0');
      timer <= (others => '0');
	   fifo_read <= '0';
	   fifo_read_q <= '0';
	   SDS_state <= "000";
	   SDS_seq <= (others => '0');
	   cell_cnt <= "00";
	   cycle_cnt <= "00";
      SDS_cnt <= (others => '0');
	   TKO_fifo_wr <= '0';
      mon_out <= (others => '0');
      partial_flag <= '0';
	   stop_by_Q <= '0';
	   stop_by_UDP <= '0';
	   start_by_UDP_p <= '0';
	   start_by_trig_p <= '0';
	   start_by_timer_p <= '0';
	   start_by_ATM_p <= '0';
	   start_by_UDP <= '0';
	   start_by_trig <= '0';
	   start_by_timer <= '0';
	   start_by_ATM <= '0';
    else	 
	   if(tick(5) = '1')then
	     en_ack <= '0';
	   elsif(UDP_rqst = '1' and FN_UDP(3) = '0' and start_TKO = '1')then
	     en_ack <= '1';
	   end if;
	   if((UDP_rqst = '1' or (fifo_read = '1' and
		    (fifo_afull = '0' or SDS_debug = '0'))) and start_TKO = '0' and
		    ((tick(5) = '1' and Q = '0') or tick = "0000000"))then
	     start_TKO <= '1';
	   else
	     start_TKO <= '0';
	   end if;
      if(tick(3) = '1' and UDP_cycle = '0')then
        got_YSSIR_n <= YSSIR;
	   end if;
	   if(tick(5) = '1' and UDP_cycle = '0')then
        got_Q <= not Q;
	   end if;
      if(tick(3) = '1' and UDP_cycle = '1')then
        got_YSSIR_UDP <= not YSSIR;
	   end if;
	   if(tick(5) = '1' and UDP_cycle = '1')then
        got_Q_UDP <= not Q;
	   end if;
	   UDP_sync <= UDP_sync(0) & UDP_TKO;
	   if(UDP_sync = "01")then
	     UDP_rqst <= '1';
	   elsif(start_TKO = '1')then
	     UDP_rqst <= '0';
	   end if;
	   if(start_TKO = '1')then
	     UDP_cycle <= UDP_rqst;
	   end if;
	   if(stop_fifo_rd_sync = "10")then
	     UDP_stop <= '1';
	   elsif(fifo_read = '0')then
	     UDP_stop <= '0';
	   end if;
	   G_trig_sync <= G_trig_sync(0) & G_trig;
	   start_fifo_rd_sync <= start_fifo_rd_sync(0) & (start_fifo_rd and SDS_ctrl(6));
	   stop_fifo_rd_sync <= stop_fifo_rd_sync(0) & stop_fifo_rd;
      tick(0) <= start_TKO;
	   tick(3 downto 1) <= tick(2 downto 0);
		tick(4) <= tick(3) or (tick(4) and not YSSIR and UDP_cycle);
	   tick(5) <= tick(4) and (YSSIR or not UDP_cycle);
	   tick(6) <= tick(5);
      if(fifo_read = '1')then
	     trig_cntr <= '0' & not trig_set;
	   elsif(G_trig_sync(1) = '0' and G_trig_sync(0) = '1' and SDS_ctrl(4) = '1')then
	     trig_cntr <= trig_cntr + 1;
	   end if;
	   if(div_time(13) = '1' or fifo_read = '1' or SDS_ctrl(5) = '0')then
	     div_time <= "00100010010000";
	   else
	     div_time <= div_time + 1;
	   end if;
      if(fifo_read = '1' or SDS_ctrl(5) = '0')then
	     timer <= '0' & (not timer_set);
	   elsif(div_time(13) = '1')then
	     timer <= timer + 1;
	   end if;
	   if(UDP_cycle = '0' and tick(5) = '1' and (Q = '1' or (UDP_stop = '1' and cell_cnt(1) = '1')))then
	     fifo_read <= '0';
	   elsif(trig_cntr(16) = '1' or timer(16) = '1' or start_fifo_rd_sync = "10" or (SDSREQ = '1' and SDS_ctrl(7) = '1'))then
	     fifo_read <= '1';
	   end if;
	   fifo_read_q <= fifo_read;
	   case SDS_state is
	     when "000" => -- waiting for SDS start
		    if(fifo_read = '1')then
		      SDS_state <= "001";
		    else
		      SDS_state <= "000";
		    end if;
	     when "001" => -- write header cell
		    if(cycle_cnt(1) = '1')then
		      SDS_state <= "010";
		    else
		      SDS_state <= "001";
		    end if;
	     when "010" => -- waiting for SDS data
		    if(fifo_read = '0')then
		      SDS_state <= "101";
		    elsif(fifo_afull = '1' and cell_cnt(1) = '1' and UDP_cycle = '0' and tick(5) = '1' and Q = '0')then
		      SDS_state <= "011";
		    else
		      SDS_state <= "010";
		    end if;
	     when "011" => -- write full warning cell
		    if(cycle_cnt(1) = '1')then
		      SDS_state <= "100";
		    else
		      SDS_state <= "011";
		    end if;
	     when "100" => -- waiting for fifo_afull go away
		    if(fifo_afull = '1')then
		      SDS_state <= "100";
		    elsif(fifo_read = '0')then
		      SDS_state <= "101"; -- go to writing trailer cell
		    elsif(cell_cnt(1) = '1' and UDP_cycle = '0' and tick(5) = '1' and Q = '0')then
		      SDS_state <= "010"; -- go back to SDS data
		    end if;
	     when "111" =>
		    if(fifo_read = '0')then
		      SDS_state <= "000";
		    else
		      SDS_state <= "001"; -- directly jump to header writing to save time
		    end if;
	     when others =>
		    SDS_state <= SDS_state + 1; -- trailer writing
	   end case;
	   if(fifo_read = '1' and (SDS_state = "000" or SDS_state = "111" or
	     (SDS_state = "100" and fifo_read_q = '0')))then
	     SDS_seq <= SDS_seq + 1;
	   end if;
	   if(fifo_read = '0')then
	     cell_cnt <= "00";
	   elsif(UDP_cycle = '0' and tick(5) = '1' and Q = '0')then
	     cell_cnt(1) <= cell_cnt(0);
	     cell_cnt(0) <= not cell_cnt(1) and not cell_cnt(0);
	   end if;
	   if(SDS_state = "001" or SDS_state = "011")then
	     cycle_cnt(1) <= cycle_cnt(0);
	     cycle_cnt(0) <= not cycle_cnt(1) and not cycle_cnt(0);
	   else
	     cycle_cnt <= "00";
	   end if;
	   if(SDS_state = "001")then
	     SDS_cnt <= (others => '0');
	   elsif(UDP_cycle = '0' and tick(5) = '1' and Q = '0')then
	     SDS_cnt <= SDS_cnt + 1;
	   end if;
	   case SDS_state is
	     when "001" | "011" | "101" | "110" | "111" => TKO_fifo_wr <= not SDS_debug;
	     when "010" => TKO_fifo_wr <= tick(5) and not Q and not UDP_cycle;
		  when others => TKO_fifo_wr <= '0';
	   end case;
-- SDS total count
	   if(UDP_cycle = '0' and tick(5) = '1' and Q = '0')then
	     mon_out(0) <= '1';
	   else	
	     mon_out(0) <= '0';
	   end if;
-- SDS total burst count
	   mon_out(1) <= fifo_read and not fifo_read_q;
-- SDS cast away count
	   if(SDS_state = "100" and UDP_cycle = '0' 
	      and tick(5) = '1' and Q = '0')then
	     mon_out(2) <= '1';
	   else	
	     mon_out(2) <= '0';
	   end if;
	   if(fifo_read_q = '1' and fifo_read = '0')then
		  partial_flag <= '0';
		elsif(SDS_state = "011" or 
		  (SDS_state = "100" and fifo_afull = '0' and cell_cnt(1) = '1' and UDP_cycle = '0' and tick(5) = '1' and Q = '0'))then
		  partial_flag <= '1';
		end if;
	   if(fifo_read_q = '1' and fifo_read = '0' and SDS_state = "100")then
-- total lost SDS burst count
	     mon_out(3) <= not partial_flag;
		else
		  mon_out(3) <= '0';
		end if;
	   if(fifo_read_q = '1' and fifo_read = '0')then
-- partial lost SDS burst count
	     mon_out(4) <= partial_flag;
		else
		  mon_out(4) <= '0';
		end if;
	   if(UDP_cycle = '0' and tick(5) = '1')then
	     stop_by_Q <= Q;
	     stop_by_UDP <= UDP_stop and cell_cnt(1);
	   end if;
	   start_by_UDP_p <= start_fifo_rd_sync(1) and not start_fifo_rd_sync(0);
		start_by_trig_p <= trig_cntr(16);
		start_by_timer_p <= timer(16);
		start_by_ATM_p <= SDSREQ and SDS_ctrl(7);
	   if(fifo_read_q = '0' and fifo_read = '1')then
	     start_by_UDP <= start_by_UDP_p;
	     start_by_trig <= start_by_trig_p;
	     start_by_timer <= start_by_timer_p;
	     start_by_ATM <= start_by_ATM_p;
	   end if;
	 end if;	
  end if;
end process;
end Behavioral;

