--	Package File Template
--
--	Purpose: This package defines supplemental types, subtypes, 
--		 constants, and functions 


library IEEE;
use IEEE.STD_LOGIC_1164.all;

package atm_pack is
constant mon_addr : std_logic_vector(15 downto 0) := x"0200";
constant SDS_BOF : std_logic_vector(11 downto 0) := x"f11";
constant SDS_EOF : std_logic_vector(11 downto 0) := x"f12";
constant SDS_OVF : std_logic_vector(11 downto 0) := x"f18";
constant SDS_trig_addr : std_logic_vector(15 downto 0) := x"0100";
constant SDS_timer_addr : std_logic_vector(15 downto 0) := x"0102";
constant SDS_ss_addr : std_logic_vector(15 downto 0) := x"0105";
constant SDS_ctrl_addr : std_logic_vector(15 downto 0) := x"0107";
constant rw_buf_addr : std_logic_vector(15 downto 0) := x"0108";
constant MAC_ctrl_addr : std_logic_vector(15 downto 0) := x"0141";
constant KEEP_INTVL_FILL_addr : std_logic_vector(15 downto 0) := x"0142";
constant KEEP_INTVL_EMPTY_addr : std_logic_vector(15 downto 0) := x"0144";
constant TOUT_ESTB_addr : std_logic_vector(15 downto 0) := x"0146";
constant TOUT_DISCNCT_addr : std_logic_vector(15 downto 0) := x"0148";
constant MSL_addr : std_logic_vector(15 downto 0) := x"014a";
constant MDIO_data_addr : std_logic_vector(15 downto 0) := x"014c";
constant MDIO_cmd_addr : std_logic_vector(15 downto 0) := x"014e";
constant TOUT_RETRANS_addr : std_logic_vector(15 downto 0) := x"0150";
constant TX_BUF_TH_addr : std_logic_vector(15 downto 0) := x"0152";
constant csr_addr : std_logic_vector(15 downto 0) := x"0001";
constant flash_buf : std_logic_vector(15 downto 0) := x"0401";
constant flash_cntr : std_logic_vector(15 downto 0) := x"0002";
constant reconf : std_logic_vector(15 downto 0) := x"0004";
constant reg_file : std_logic_vector(15 downto 0) := x"0100";
constant my_TCP_port : std_logic_vector(15 downto 0) := x"0017";
constant my_UDP_port : std_logic_vector(15 downto 0) := x"1234";
constant my_MSS : std_logic_vector(11 downto 0) := x"5B4";
constant my_IP_MSB : std_logic_vector(19 downto 0) := x"80C52";
constant TX_fifo_SA : std_logic_vector(11 downto 0) := x"000";
constant RX_fifo_SA : std_logic_vector(11 downto 0) := x"000";
constant N : integer := 24; -- sdram fifo size
component ibufgds
    port (i : in std_logic; ib : in std_logic; o : out std_logic);
end component;
component bufg
    port (i : in std_logic; o : out std_logic);
end component;
component DCM
    generic ( CLK_FEEDBACK	: string := "1X";
	 			  CLKDV_DIVIDE : real := 2.0;
				  CLKFX_DIVIDE : integer := 1;
				  CLKFX_MULTIPLY : integer := 4;
				  CLKIN_DIVIDE_BY_2 : boolean := FALSE;
	 			  CLKIN_PERIOD : real := 0.0;
	 			  CLKOUT_PHASE_SHIFT : string := "NONE";
      		  DESKEW_ADJUST : string := "SYSTEM_SYNCHRONOUS";
		        DFS_FREQUENCY_MODE : string := "LOW";
      		  DLL_FREQUENCY_MODE : string := "LOW";
	 			  DSS_MODE : string := "NONE";
      		  DUTY_CYCLE_CORRECTION : boolean := TRUE;
				  PHASE_SHIFT : integer := 0;
      		  STARTUP_WAIT : boolean := FALSE);
    port ( CLKIN   : in  std_logic ;
           CLKFB   : in  std_logic ;
           RST     : in  std_logic ;
           DSSEN   : in  std_logic ;
           PSINCDEC: in  std_logic ;
           PSEN    : in  std_logic ;
           PSCLK   : in  std_logic ;
           CLK0    : out std_logic ;
           CLK90   : out std_logic ;
           CLK180  : out std_logic ;
           CLK270  : out std_logic ;
           CLK2X   : out std_logic ;
           CLK2X180: out std_logic ;
           CLKDV   : out std_logic ;
           CLKFX   : out std_logic ;
           CLKFX180: out std_logic ;
           LOCKED  : out std_logic ;
           STATUS  : out std_logic_vector(7 downto 0);
           PSDONE  : out std_logic );
end component;
component RAMB16_S9_S18
-- pragma translate_off
     generic(
         write_mode_a: string := "NO_CHANGE"; -- write_first(default)/read_first/no_change
         init_b: bit_vector(17 downto 0) := "000000000000000000";
         srval_b: bit_vector(17 downto 0) := "000000000000000000";
         write_mode_b: string := "NO_CHANGE"; -- write_first(default)/read_first/no_change
         init_a: bit_vector(8 downto 0) := "000000000";
         srval_a: bit_vector(8 downto 0) := "000000000");
-- pragma translate_on
     port(
         dia: in std_logic_vector(7 downto 0); 
         dipa: in std_logic_vector(0 downto 0); 
         addra: in std_logic_vector(10 downto 0);
         clka: in std_logic; 
         ssra: in std_logic; 
         ena: in std_logic; 
         wea: in std_logic; 
         doa: out std_logic_vector(7 downto 0); 
         dopa: out std_logic_vector(0 downto 0); 
         dib: in std_logic_vector(15 downto 0); 
         dipb: in std_logic_vector(1 downto 0); 
         addrb: in std_logic_vector(9 downto 0);
         clkb: in std_logic; 
         ssrb: in std_logic; 
         enb: in std_logic; 
         web: in std_logic; 
         dob: out std_logic_vector(15 downto 0); 
         dopb: out std_logic_vector(1 downto 0));
end component;
---------------------------------
component ram16XNd
    generic ( N: positive := 8);
    port (
        wclk: in STD_LOGIC;
        rclk: in STD_LOGIC;
        we: in STD_LOGIC;
        d: in STD_LOGIC_VECTOR (N-1 downto 0);
        wa: in STD_LOGIC_VECTOR (3 downto 0);
        ra: in STD_LOGIC_VECTOR (3 downto 0);
        q: out STD_LOGIC_VECTOR (N-1 downto 0)
    );
end component;
component delayN
    generic (N: positive := 2;
	          INIT : bit_vector := x"0000");
    port (
        c: in STD_LOGIC;
        d: in STD_LOGIC;
        ce: in STD_LOGIC;
        q: out STD_LOGIC
    );
end component;
---------------------------------------------------------------------------------
component ddr_io
    port (
        clk: in STD_LOGIC;
        clkn: in STD_LOGIC;
        pad: inout std_logic;
        t: in STD_LOGIC;
        d0: in STD_LOGIC;
        d1: in STD_LOGIC;
        iclk: in STD_LOGIC;
        iclkn: in STD_LOGIC;
        q0: out std_logic; -- rising edge       
        q1: out std_logic  -- falling edge    
    );
end component;
---------------------------------------------------------------------------------
COMPONENT SSN_if
   GENERIC (step : integer := 1);
	PORT(
		clk : IN std_logic;
		en_SSN : IN std_logic;
		SSN : INOUT std_logic;      
		SSN_ROM : OUT std_logic_vector(63 downto 0);
		SSN_CRC : OUT std_logic_vector(7 downto 0);
		SSN_valid : OUT std_logic;
		SSN_done : OUT std_logic
		);
END COMPONENT;
---------------------------------------------------------
COMPONENT flash
	PORT(
		clk : IN std_logic;
		reset : IN std_logic;
      sel_SPI : in  STD_LOGIC;
		re : IN std_logic;
		we : IN std_logic;
		wdata : IN std_logic_vector(7 downto 0);
		addr : IN std_logic_vector(15 downto 0);
		MQ : IN std_logic;          
		rdata : OUT std_logic_vector(7 downto 0);
		ack : OUT std_logic;
		reconf_b : OUT std_logic;
		MD : OUT std_logic;
		MW_n : OUT std_logic;
		MCS : OUT std_logic;
      SPI_SO : in  STD_LOGIC;
      SPI_SI : out  STD_LOGIC;
      SPI_SCK : out  STD_LOGIC;
      SPI_CSn : out  STD_LOGIC
		);
END COMPONENT;
---------------------------------------------------------
component SiTCP_FOR_SK
   port(
-- System
	CLK : in std_logic; -- in	: System clock (>12.5MHz)
	RSTn : in std_logic; -- in	: System reset
-- For debug
	VERSION : out std_logic_vector(31 downto 0); -- out	: Version # of SiTCP
-- MAC and IP address
	MY_TCP_PORT : in std_logic_vector(15 downto 0); -- in	: TCP data port #[15:0]
	MY_MAC_ADDR : in std_logic_vector(47 downto 0); -- in	: MAC address[47:0]
	MY_IP_ADDR : in std_logic_vector(31 downto 0); -- in	: IP address[31:0]
	MY_MSS : in std_logic_vector(11 downto 0); -- in	: MSS [11:0]
-- MII interface
	MII_MAC_FLOW_ENB : in std_logic; -- in	: Enable of MAC flow control(1:Enable 0:Disable)
	KEEP_ON			 : in std_logic; -- in	: Keep alive-on (1:ON, 0:OFF)
	FAST_RETRANS_ON	: in std_logic; -- in : Fast retransmit
	KEEP_INTVL_FILL	 : in std_logic_vector(15 downto 0); -- in	: Interval time to sent keep-alive-packets when a tx-buffer is not empty. [15:0] (msec)
	KEEP_INTVL_EMPTY : in std_logic_vector(15 downto 0); -- in	: Interval time to sent keep-alive-packets when a tx-buffer is empty. [15:0] (msec)
	TOUT_ESTB		 : in std_logic_vector(15 downto 0); -- in	: Time-out in establish phase (From recved SYN to receive the next ACK). [15:0] (msec)
	TOUT_DISCNCT	 : in std_logic_vector(15 downto 0); -- in	: Disconnect time-out when a connection is established. [15:0] (msec)
	TOUT_RETRANS		 : in std_logic_vector(15 downto 0); -- in	: Time-out in retransmit [15:0] (microsec)
	MSL				 : in std_logic_vector(15 downto 0); -- in	: Min. interval time to allow to connect the next connection. [15:0] (msec)	
	TX_BUF_TH		 : in std_logic_vector(15 downto 0); -- in	: Transmit buffer threshold [15:0]
--	 TX
	MII_TX_CLK : in std_logic; -- in	: Tx clock(2.5 or 25MHz)
	MII_TX_EN : out std_logic; -- out	: Tx enable
	MII_TXD : out std_logic_vector(3 downto 0); -- out	: Tx data[3:0]
	MII_TX_ER : out std_logic; -- out	: TX error
-- RX
	MII_RX_CLK : in std_logic; -- in	: Rx clock(2.5 or 25MHz)
	MII_RX_DV : in std_logic; -- in	: Rx data valid
	MII_RXD : in std_logic_vector(3 downto 0); -- in	: Rx data[3:0]
	MII_RX_ER : in std_logic; -- in	: Rx error
	MII_CRS : in std_logic; -- in	: Carrier sense
	MII_COL : in std_logic; -- in	: Collision detected
-- Management IF
	MII_MDC : out std_logic; -- out	: Clock for MDIO
	MII_MDIO : inout std_logic; -- io	: data
-- UDP
	MY_UDP_PORT : in std_logic_vector(15 downto 0); -- in	: UDP port #, For Remote Bus Controller
-- Local bus interface
	LOC_ADDR : out std_logic_vector(31 downto 0); -- out	: Address[31:0]
	LOC_WE : out std_logic; -- out	: Write enable
	LOC_WD : out std_logic_vector(7 downto 0); -- out	: Data[7:0]
	LOC_RE : out std_logic; -- out	: Read enable
	LOC_ACK : in std_logic; -- in	: Access acknowledge
	LOC_RD : in std_logic_vector(7 downto 0); -- in	: Read data[7:0]
-- TCP connection control
	TCP_ESTABLISH : out std_logic; -- out	: Connection close
	TCP_CLOSE_REQ : out std_logic; -- out	: Connection close
	TCP_CLOSE_ACK : in std_logic; -- in	: Acknowledge for close
-- FIFO I/F
	RX_FIFO_FILL : in std_logic_vector(15 downto 0); -- in	: Fill level[15:0]
	RX_FIFO_WR : out std_logic; -- out	: Write enable
	RX_FIFO_DATA : out std_logic_vector(7 downto 0); -- out	: Write data[7:0]
	TX_FIFO_AFULL : out std_logic; -- out	: Almost full flag
	TX_FIFO_WR : in std_logic; -- in	: Write enable
	TX_FIFO_DATA : in std_logic_vector(7 downto 0) -- in	: Write data[7:0]
);
end component;
---------------------------------------------------------
component refresh
    Port ( clk : in std_logic;
           lock : in std_logic;
           init_done : in std_logic;
           start_init : out std_logic;
           ddr_csn : out std_logic;
           ddr_clke : out std_logic;
           ref : out std_logic);
end component;
-----------------------------------------------------------------
COMPONENT sdram_fifo
	PORT(
		reset : IN std_logic;
		clk : IN std_logic;
		TKO_clk : IN std_logic;
		clk4x : IN std_logic;
		mem_status : OUT std_logic_vector(15 downto 0);
		not_locked : IN std_logic;
		mem_test : IN std_logic;
		TKO_data : IN std_logic_vector(15 downto 0);
		TKO_wr : IN std_logic;
		TCP_byte_order : in std_logic;
		TCP_ESTABLISH : IN std_logic;
		TCP_CLOSE_REQ : IN std_logic;
		TX_fifo_afull : IN std_logic;    
		mdq : INOUT std_logic_vector(3 downto 0);
		dqs : INOUT std_logic;      
		fifo_afull : OUT std_logic;
		TCP_CLOSE_ACK : OUT std_logic;
		TX_fifo_data : OUT std_logic_vector(7 downto 0);
		TX_fifo_we : OUT std_logic;
		maddr : OUT std_logic_vector(12 downto 0);
		mba : OUT std_logic_vector(1 downto 0);
		dm : OUT std_logic;
		we_n : OUT std_logic;
		ras_n : OUT std_logic;
		cas_n : OUT std_logic;
		cs_n : OUT std_logic;
		mclke : OUT std_logic;
		mclk : OUT std_logic;
		mclk_n : OUT std_logic
		);
END COMPONENT;
COMPONENT TKO_if
	PORT(
		clk : IN std_logic;
		clk60 : IN std_logic;
		reset : IN std_logic;
		rst_err : IN std_logic;
		fifo_afull : IN std_logic;
		SDSREQ : IN std_logic;
      SDS_debug : in  STD_LOGIC;
		RSV : IN std_logic_vector(1 downto 0);
		Q : IN std_logic;
		YSSIR : IN std_logic;
		G_TRIG : IN std_logic;
		addr : IN std_logic_vector(31 downto 0);
		wd : IN std_logic_vector(7 downto 0);
		we : IN std_logic;
		re : IN std_logic;    
		D : INOUT std_logic_vector(15 downto 0);      
		FN : OUT std_logic_vector(3 downto 0);
		SA : OUT std_logic_vector(10 downto 0);
		DOIT : OUT std_logic;
		TKO_ack : OUT std_logic;
		mon_out : OUT std_logic_vector(4 downto 0);
		ATM_data : OUT std_logic_vector(7 downto 0);
		TKO_data : OUT std_logic_vector(7 downto 0);
		SDS_seq_o : OUT std_logic_vector(35 downto 0);
		TKO_fifo_wr : OUT std_logic;
		TKO_fifo_data : OUT std_logic_vector(15 downto 0)
		);
END COMPONENT;
COMPONENT monitor
	PORT(
		clk : IN std_logic;
		clk4x : IN std_logic;
		TKO_clk : IN std_logic;
		reset : IN std_logic_vector(15 downto 0);
		latch : IN std_logic_vector(15 downto 0);
		addr : IN std_logic_vector(15 downto 0);
		re : IN std_logic;
		we : IN std_logic;
		mon_in : IN std_logic_vector(15 downto 0);          
		ack : OUT std_logic;
		dout : OUT std_logic_vector(7 downto 0)
		);
END COMPONENT;
COMPONENT crc16D8
	PORT(
		clk : IN std_logic;
		init_crc : IN std_logic;
		we_crc : IN std_logic;
		d : IN std_logic_vector(7 downto 0);          
		crc : OUT std_logic_vector(15 downto 0)
		);
END COMPONENT;
COMPONENT crc16D16
	PORT(
		clk : IN std_logic;
		init_crc : IN std_logic;
		we_crc : IN std_logic;
		d : IN std_logic_vector(15 downto 0);          
		crc : OUT std_logic_vector(15 downto 0)
		);
END COMPONENT;
end atm_pack;
package body atm_pack is
end atm_pack;
