----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    15:39:40 06/29/2006 
-- Design Name: 
-- Module Name:    flash - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
-- flash buffer starts at 0x400
-- FPGA conf data must be in address ranges 0x600-0x7ff
-- once flash buffer filled with data, write number of bytes
-- to be sent to the FLASH to address 0x1, this starts the
-- 
----------------------------------------------------------------------------------
library IEEE,mylib;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use mylib.atm_pack.all;
---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity flash is
    Port ( clk : in  STD_LOGIC;
           reset : in  STD_LOGIC;
           sel_SPI : in  STD_LOGIC;
           re : in  STD_LOGIC;
           we : in  STD_LOGIC;
           wdata : in  STD_LOGIC_VECTOR (7 downto 0);
           rdata : out  STD_LOGIC_VECTOR (7 downto 0);
           addr : in  STD_LOGIC_VECTOR (15 downto 0);
           ack : out  STD_LOGIC;
           reconf_b : out  STD_LOGIC;
           MQ : in  STD_LOGIC;
           MD : out  STD_LOGIC;
           MW_n : out  STD_LOGIC;
           MCS : out  STD_LOGIC;
-- ATM board SPI control
           SPI_SO : in  STD_LOGIC;
           SPI_SI : out  STD_LOGIC;
           SPI_SCK : out  STD_LOGIC;
           SPI_CSn : out  STD_LOGIC
			  );
end flash;

architecture Behavioral of flash is
signal sel_SPI_n : std_logic;
signal SPI_SCK_i : std_logic;
signal dia : std_logic_vector(0 downto 0);
signal doa : std_logic_vector(0 downto 0);
signal dib : std_logic_vector(7 downto 0);
signal dob : std_logic_vector(7 downto 0);
signal addra : std_logic_vector(13 downto 0);
signal addrap : std_logic_vector(13 downto 0);
signal addrb : std_logic_vector(10 downto 0);
signal byte_cnt : std_logic_vector(8 downto 0);
signal en_FLASH : std_logic := '0';
signal en_FLASH_dl : std_logic_vector(1 downto 0) := "00";
signal ce_addra_dl : std_logic_vector(6 downto 0) := "0000000";
signal ce_addra : std_logic := '0';
signal illegal : std_logic := '0';
signal READ : std_logic := '0';
signal wea : std_logic := '0';
signal web : std_logic := '0';
signal re_conf : std_logic := '0';
signal backup : std_logic := '0';
signal we_wait : std_logic;
attribute INIT : string;
attribute INIT of re_conf,en_flash : signal is "0";
begin
MW_n <= '1';
SPI_SCK <= SPI_SCK_i;
sel_SPI_n <= not sel_SPI;
reconf_b <= not re_conf;
rdata <= dob when addr(9) = '1' else dob(0) & dob(1) & dob(2) & dob(3) &
                                     dob(4) & dob(5) & dob(6) & dob(7);
process(clk)
begin
  if(clk'event and clk = '1')then
    if(addr(9) = '1')then
	   dib <= wdata;
-- if not FPGA configuration data, invert bit order
	 else
	   for i in 0 to 7 loop
		  dib(i) <= wdata(7-i);
		end loop;
	 end if;
	 if(en_flash = '0')then
	   we_wait <= '0';
	 elsif(we = '1' and addr(15 downto 10) = flash_buf(15 downto 10))then
	   we_wait <= '1';
	 end if;
	 if(we_wait = '1')then
	   ack <= not en_flash;
	 elsif(addr(15 downto 10) = flash_buf(15 downto 10))then
	   ack <= (we and not en_flash) or re;
	 elsif(we = '1' and (addr(15 downto 1) = flash_cntr(15 downto 1) or addr(15 downto 1) = reconf(15 downto 1)))then
	   ack <= '1';
	 else
	   ack <= '0';
	 end if;
	 if(addr(15 downto 10) = flash_buf(15 downto 10))then
	   web <= we;
	 else
	   web <= '0';
	 end if;
	 if(addr = flash_buf and we = '1')then
	   case wdata( 3 downto 0) is
		  when x"3" => READ <= '1';
		  when others => READ <= '0'; 
		end case;
	   case wdata( 3 downto 0) is
		  when x"7" | x"9" | x"b" => illegal <= '1';
		  when others => illegal <= '0'; 
		end case;
	 end if;
	 if(en_FLASH = '0')then
		addra <= (others => '0');
	 elsif(ce_addra = '1')then
		addra(11 downto 0) <= addra(11 downto 0) + 1;
	 end if;
	 if(en_FLASH_dl(1) = '0')then
		addrap <= (others => '0');
	 elsif(wea = '1')then
		addrap(11 downto 0) <= addrap(11 downto 0) + 1;
	 end if;
-- FLASH signals
	 if(addr(15 downto 1) = flash_cntr(15 downto 1) and addr(0) = '1' and we = '1' and illegal = '0')then
	   en_FLASH <= '1';
	 elsif(addra(11 downto 3) = byte_cnt and addra(2 downto 0) = "111" and ce_addra = '1')then
	   en_FLASH <= '0';
	 end if;
	 SPI_CSn <= not en_FLASH_dl(0) or not sel_SPI;
	 MCS <= en_FLASH_dl(0) and not sel_SPI;
	 en_FLASH_dl <= en_FLASH_dl(0) & en_FLASH;
	 if(en_FLASH = '0')then
	   ce_addra <= '0';
	 else
	   ce_addra <= not ce_addra;
	 end if;
	 SPI_SCK_i <= not SPI_SCK_i and sel_SPI and en_FLASH_dl(1);
	 SPI_SI <= doa(0) and sel_SPI;
	 MD <= (re_conf and backup) or (not re_conf and doa(0) and not sel_SPI);
	 if(addr(15 downto 1) = reconf(15 downto 1) and we = '1')then
	   if(addr(0) = '0')then
	     backup <= wdata(0);
	   elsif(wdata = x"a5")then
	     re_conf <= '1';
	   end if;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(we = '1' and addr(15 downto 1) = flash_cntr(15 downto 1))then
	   if(addr(0) = '1')then
	     byte_cnt(7 downto 0) <= wdata;
		else
	     byte_cnt(8) <= wdata(0);
		end if;
	 end if;
	 ce_addra_dl(1) <= ce_addra_dl(0);
	 ce_addra_dl(3) <= ce_addra_dl(2);
	 ce_addra_dl(5) <= ce_addra_dl(4);
	 wea <= ce_addra_dl(6) and en_FLASH_dl(1);
  end if;
end process;
dia(0) <= MQ when sel_SPI = '0' else SPI_SO;
addrb <= "00" & addr(8 downto 1) & not addr(0);
i_wbuf: ramb16_s1_s9
     port map(
         dia => "0", 
         addra => addra,
         clka => clk, 
         ssra => '0', 
         ena => '1', 
         wea => '0',
         dib => dib,
         dipb => "0",
         addrb => addrb,
         clkb => clk,
         ssrb => '0',
         enb => '1',
         web => web,
			doa => doa);
i_rbuf: ramb16_s1_s9
     port map(
         dia => dia, 
         addra => addrap,
         clka => clk, 
         ssra => '0', 
         ena => '1', 
         wea => wea,
         dib => x"00",
         dipb => "0",
         addrb => addrb,
         clkb => clk,
         ssrb => '0',
         enb => '1',
         web => '0',
         dob => dob);
i_SRLC16a : SRL16
   port map (
      Q => ce_addra_dl(0),
      A0 => '0',
      A1 => sel_SPI_n,
      A2 => READ,
      A3 => READ,
      CLK => CLK,
      D => ce_addra
   );
i_SRL16b : SRL16
   port map (
      Q => ce_addra_dl(2),
      A0 => '1',
      A1 => '1',
      A2 => READ,
      A3 => READ,
      CLK => CLK,
      D => ce_addra_dl(1)
   );
i_SRL16c : SRL16
   port map (
      Q => ce_addra_dl(4),
      A0 => '1',
      A1 => '1',
      A2 => READ,
      A3 => READ,
      CLK => CLK,
      D => ce_addra_dl(3)
   );
i_SRL16d : SRL16
   port map (
      Q => ce_addra_dl(6),
      A0 => '1',
      A1 => '1',
      A2 => READ,
      A3 => READ,
      CLK => CLK,
      D => ce_addra_dl(5)
   );

end Behavioral;

