library IEEE,mylib;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use mylib.atm_pack.all;
library UNISIM;
use UNISIM.VComponents.all;

entity sdram_fifo is
    port (
-- reset input, should be connected to the inverted LOCK output of dll
        reset: in STD_LOGIC;
-- system clock input
        clk: in STD_LOGIC;
        TKO_clk: in STD_LOGIC;
        clk4x: in STD_LOGIC;
        mem_status : out  STD_LOGIC_VECTOR (15 downto 0);
-- DLL lock input
        not_locked: in STD_LOGIC;
        mem_test: in STD_LOGIC;
-- TKO interface signals
        TKO_data : in  STD_LOGIC_VECTOR (15 downto 0);
        TKO_wr : in  STD_LOGIC;
        fifo_afull : out  STD_LOGIC;
-- SiTCP interface signals
		  TCP_byte_order : in std_logic;
		  TCP_ESTABLISH : in std_logic;
		  TCP_CLOSE_REQ : in std_logic;
		  TCP_CLOSE_ACK : out std_logic;
        TX_fifo_data : out  STD_LOGIC_VECTOR (7 downto 0);
        TX_fifo_we : out  STD_LOGIC;
        TX_fifo_afull : in  STD_LOGIC;
-- ram pads
		  mdq : inout STD_LOGIC_VECTOR (3 downto 0);
		  maddr : out STD_LOGIC_VECTOR (12 downto 0);
		  mba : out STD_LOGIC_VECTOR (1 downto 0);
		  dm : out STD_LOGIC;
		  dqs : inout STD_LOGIC;
		  we_n : out STD_LOGIC;
		  ras_n : out STD_LOGIC;
		  cas_n : out STD_LOGIC;
		  cs_n : out STD_LOGIC;
		  mclke : out STD_LOGIC;
		  mclk : out STD_LOGIC;
		  mclk_n : out STD_LOGIC
    );
end sdram_fifo;

architecture sdram_fifo_arch of sdram_fifo is
-- sdram i/o
signal ma: std_logic_vector(14 downto 0);
signal md: std_logic_vector(7 downto 0);
signal mq: std_logic_vector(7 downto 0);
signal mq_q: std_logic_vector(7 downto 0);
signal rasn: std_logic;
signal casn: std_logic;
signal wen: std_logic;
signal t_md: std_logic;
signal t_dqs: std_logic;
-- refresh signal generation
signal ref: std_logic;
signal start_init: std_logic;
-- sdram state machine
signal is_read: std_logic;
signal rqst: std_logic;
signal rd_rqst: std_logic;
signal en_dll: std_logic := '0';
signal rst_dll: std_logic := '0';
signal w_cycle: std_logic;
signal r_cycle: std_logic;
signal rd_burst: std_logic;
signal rd_burstp: std_logic;
signal stop_w: std_logic;
signal stop_w_q: std_logic;
signal stop_r: std_logic;
signal inh_ras: std_logic;
signal ref_rqst: std_logic;
signal init_done: std_logic_vector(1 downto 0);
signal pc_dl: std_logic;
signal precharge: std_logic;
signal prechage_dl: std_logic;
signal rfc_cntr: std_logic_vector(3 downto 0) := x"0";
-- random memory test signals
signal lfsr : std_logic_vector(15 downto 0) := x"0000";
signal div7: std_logic_vector(2 downto 0) := "000";
signal ec_lfsr: std_logic;
signal mclk0_dcm: std_logic;
signal mclk90_dcm: std_logic;
signal mclk180_dcm: std_logic;
signal mclk270_dcm: std_logic;
signal mclk0: std_logic;
signal mclk90: std_logic;
signal mclk180: std_logic;
signal mclk270: std_logic;
signal lock: std_logic;
signal clk4xn: std_logic;
constant zero : std_logic_vector(N-1 downto 0) := (others => '0');
signal fifo_wc : std_logic_vector(N-1 downto 0);
signal fifo_wa : std_logic_vector(N-1 downto 0);
signal fifo_ra : std_logic_vector(N-1 downto 0);
signal fifo_afull_i: std_logic;
signal fifo_afull_sync: std_logic;
signal fifo_wr: std_logic;
signal TKO_we: std_logic;
signal TKO_wa : std_logic_vector(9 downto 0);
signal TKO_ra : std_logic_vector(10 downto 0);
signal TKO_wc : std_logic_vector(7 downto 0);
signal TKO_dout : std_logic_vector(7 downto 0);
signal TKO_dout_q : std_logic_vector(7 downto 0);
signal TKO_din : std_logic_vector(15 downto 0);
signal TKO_wa_sync : std_logic_vector(2 downto 0);
signal TCP_wa : std_logic_vector(10 downto 0);
signal TCP_wap : std_logic_vector(7 downto 0);
signal TCP_wa_gray : std_logic_vector(2 downto 0);
signal TCP_wa_sync : std_logic_vector(3 downto 0);
signal TCP_ra_sync : std_logic_vector(1 downto 0);
signal TCP_wc : std_logic_vector(1 downto 0);
signal TCP_ra : std_logic_vector(10 downto 0);
signal dqs_in : std_logic_vector(1 downto 0);
signal dqs_in_q : std_logic_vector(1 downto 0);
signal dqs_status : std_logic_vector(3 downto 0);
signal TCP_wa6q: std_logic := '0';
signal TCP_wr: std_logic := '0';
signal psincdec: std_logic := '0';
signal psen: std_logic := '0';
signal psdone: std_logic := '0';
signal ps_rdy: std_logic := '0';
signal ps_cntr : std_logic_vector(7 downto 0);
signal ps_tgl: std_logic := '0';
signal ps_tgl_sync : std_logic_vector(1 downto 0);
signal TX_fifo_we_i: std_logic;
signal dqs_out: std_logic;
signal mem_rdy: std_logic;
signal en_mem_rdy: std_logic;
signal mem_rdyp: std_logic;
signal en_TCP: std_logic;
signal tcp_empty: std_logic;
begin
process(clk)
begin
  if(clk'event and clk = '1')then
    if(ps_tgl_sync(1) /= ps_tgl_sync(0))then
      mem_status(7 downto 0) <= ps_cntr;
	 end if;
  end if;
end process;  
tcp_empty <= '1' when TCP_ra(8 downto 1) = TCP_wap else '0';
mem_status(15 downto 8) <= mem_rdy & init_done(1) & ps_rdy & TCP_empty & "000" & TKO_we;
fifo_afull <= fifo_afull_sync;
TX_fifo_we <= TX_fifo_we_i;
clk4xn <= not clk4x;
i_mclk:  fddrrse
    port map(
          d0 => '1',
          d1 => '0',
          c0 => clk4xn,
          c1 => clk4x,
          ce => '1',
          r  => '0',
          s  => '0',
          q  => mclk);
i_mclk_n:  fddrrse
    port map(
          d0 => '0',
          d1 => '1',
          c0 => clk4xn,
          c1 => clk4x,
          ce => '1',
          r  => '0',
          s  => '0',
          q  => mclk_n);
DCMmclk: dcm 
   generic map(CLKOUT_PHASE_SHIFT => "VARIABLE",
					CLKIN_PERIOD => 10.0,
					PHASE_SHIFT => 0)  
	port map	
         ( CLKIN   => clk4x ,
           CLKFB   => mclk90 ,
           RST     => not_locked ,
           DSSEN   => '0' ,
           PSINCDEC=> psincdec ,
           PSEN    => psen ,
           PSCLK   => mclk0 ,
           PSDONE  => psdone ,
           CLK0    => mclk90_dcm ,
			  clk90  => mclk180_dcm,
           CLK270  => mclk0_dcm ,
           LOCKED  => lock);
mclk270 <= not mclk90;
mclk0bufg : BUFG port map (O=>mclk0, I=>mclk0_dcm);
mclk90bufg : BUFG port map (O=>mclk90, I=>mclk90_dcm);
mclk180bufg : BUFG port map (O=>mclk180, I=>mclk180_dcm);
i_rd_burstp: delayN
  generic map(N=> 4) port map(d => rd_burst, ce => '1', c => mclk180, q => rd_burstp);
process(mclk0,lock)
begin
  if(lock = '0')then
    mem_rdy <= '0';
    en_mem_rdy <= '0';
    ps_rdy <= '1';
    ps_cntr <= x"80";
	 dqs_status <= x"0";
	 psen <= '0';
	 psincdec <= '0';
  elsif(mclk0'event and mclk0 = '1')then
    ps_rdy <= not psen and (ps_rdy or psdone);
	 if(psen = '1')then
	   if(psincdec = '1')then
	     ps_cntr <= ps_cntr + 1;
	   else
	     ps_cntr <= ps_cntr - 1;
	   end if;
	 end if;
	 if(psdone = '1' or ref = '1')then
	   dqs_status <= x"0";
	 elsif(TCP_wr = '1')then
	   dqs_status(0) <= dqs_status(0) or not dqs_in_q(0);
	   dqs_status(1) <= dqs_status(1) or dqs_in_q(0);
	   dqs_status(2) <= dqs_status(2) or not dqs_in_q(1);
	   dqs_status(3) <= dqs_status(3) or dqs_in_q(1);
		en_mem_rdy <= '1';
	 end if;
	 if(ref = '1' and ps_rdy = '1')then
	   case dqs_status is
--		  when "0000" => null;	 
		  when "1001" =>
		    psen <= '1';
--		    psen <= not mem_rdy;
			 psincdec <= '1';
			 if(en_mem_rdy = '1' and psincdec = '0')then
			   mem_rdy <= '1';
			 end if;
		  when "0110" =>
		    psen <= '1';
--		    psen <= not mem_rdy;
			 psincdec <= '0';
			 if(en_mem_rdy = '1' and psincdec = '1')then
			   mem_rdy <= '1';
			 end if;
--		  when others => mem_rdy <= '1';	 
		  when others => null;	 
		end case;
	 else
	   psen <= '0';
	 end if;
  end if;
end process;
process(mclk0)
begin
  if(mclk0'event and mclk0 = '1')then
    dqs_in_q <= dqs_in;
	 ps_tgl <= ps_tgl xor psen;
  end if;
end process;
-- ram pads
dm <= '0';
i_dqs: ddr_io
    port map(
          clk => mclk270,
          clkn => mclk90,
          pad => dqs,
			 t => t_dqs,
          d0 => dqs_out,
          d1 => '0',
          iclk => mclk0,
          iclkn => mclk180,
          q0 => dqs_in(0),
          q1 => dqs_in(1));
process(mclk0)
begin
  if(mclk0'event and mclk0 = '1')then
    we_n <= wen;
    cas_n <= casn;
    ras_n <= rasn;
    maddr <= ma(12 downto 0);
	 mba <= ma(14 downto 13);
  end if;
end process;
g_mdq: for i in 0 to 3 generate
  ddr_data: ddr_io
    port map(
          clk => mclk0,
          clkn => mclk180,
          pad => mdq(i),
			 t => t_md,
          d0 => md(4+i),
          d1 => md(i),
          iclk => mclk90,
          iclkn => mclk270,
          q0 => mq(i),
          q1 => mq(4+i));
end generate g_mdq;
process(mclk90)
begin
  if(mclk90'event and mclk90 = '1')then
    mq_q <= mq;
  end if;
end process;
process(mclk0)
begin
  if(mclk0'event and mclk0 = '1')then
	 TKO_dout_q <= TKO_dout;
	 md(7 downto 4) <= TKO_dout_q(7 downto 4);
  end if;
end process;
process(mclk180)
begin
  if(mclk180'event and mclk180 = '1')then
	 md(3 downto 0) <= TKO_dout_q(3 downto 0);
  end if;
end process;
----------------------------------------------------------------------------------------
process(TKO_clk,init_done)
begin
  if(init_done(1) = '0')then
    TKO_we <= '0';
  elsif(TKO_clk'event and TKO_clk = '1')then
    if(mem_test = '0')then
		TKO_we <= TKO_wr;
	 else
		TKO_we <= ec_lfsr;
	 end if;	
  end if;
end process;  
-- random memory test
process(TKO_clk)
begin
  if(TKO_clk'event and TKO_clk = '1')then
    fifo_afull_sync <= fifo_afull_i;
    if(mem_test = '0')then
	   lfsr <= (others => '0');
	 elsif(ec_lfsr = '1')then
	   lfsr <= lfsr(14 downto 0) & not(lfsr(15) xor lfsr(14) xor lfsr(12) xor lfsr(3));
	 end if;
    if(mem_test = '0')then
	   if(TCP_byte_order = '0')then
	     TKO_din <= TKO_data(7 downto 0) & TKO_data(15 downto 8);
		else
	     TKO_din <= TKO_data;
		end if;
		div7 <= "000";
		ec_lfsr <= '0';
	 else
	   if(TCP_byte_order = '0')then
	     TKO_din <= lfsr(7 downto 0) & lfsr(15 downto 8);
		else
	     TKO_din <= lfsr;
		end if;
		if(div7 = "110")then
		  div7 <= "000";
		  ec_lfsr <= not fifo_afull_sync;
		else
		  div7 <= div7 + 1;
		  ec_lfsr <= '0';
		end if;
	 end if;
  end if;
end process;
i_TKO_fifo : RAMB16_S9_S18
   port map (
      DOA => TKO_dout,      -- Port A 8-bit Data Output
      ADDRA => TKO_ra,  -- Port A 11-bit Address Input
      ADDRB => TKO_wa,  -- Port B 10-bit Address Input
      CLKA => mclk180,    -- Port A Clock
      CLKB => TKO_clk,    -- Port B Clock
      DIA => x"00",      -- Port A 8-bit Data Input
      DIB => TKO_din,      -- Port B 16-bit Data Input
      DIPA => "0",    -- Port A 1-bit parity Input
      DIPB => "00",    -- Port-B 2-bit parity Input
      ENA => '1',      -- Port A RAM Enable Input
      ENB => '1',      -- PortB RAM Enable Input
      SSRA => '0',    -- Port A Synchronous Set/Reset Input
      SSRB => '0',    -- Port B Synchronous Set/Reset Input
      WEA => '0',      -- Port A Write Enable Input
      WEB => TKO_we    -- Port B Write Enable Input
   );
i_TCP_fifo : RAMB16_S9_S9
   port map (
      DOA => TX_fifo_data,      -- Port A 8-bit Data Output
      ADDRA => TCP_ra,  -- Port A 11-bit Address Input
      ADDRB => TCP_wa,  -- Port B 10-bit Address Input
      CLKA => clk,    -- Port A Clock
      CLKB => mclk90,    -- Port B Clock
      DIA => x"00",      -- Port A 8-bit Data Input
      DIB => mq_q,      -- Port B 16-bit Data Input
      DIPA => "0",    -- Port A 1-bit parity Input
      DIPB => "0",    -- Port-B 2-bit parity Input
      ENA => '1',      -- Port A RAM Enable Input
      ENB => '1',      -- PortB RAM Enable Input
      SSRA => '0',    -- Port A Synchronous Set/Reset Input
      SSRB => '0',    -- Port B Synchronous Set/Reset Input
      WEA => '0',      -- Port A Write Enable Input
      WEB => TCP_wr    -- Port B Write Enable Input
   );
process(TKO_clk,reset)
begin
  if(reset = '1')then
    TKO_wa <= (others => '0');
  elsif(TKO_clk'event and TKO_clk = '1')then
    if(TKO_we = '1')then
	   TKO_wa(7 downto 0) <= TKO_wa(7 downto 0) + 1;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   ps_tgl_sync <= "00";
      TCP_wa_sync <= (others => '0');
      TCP_wap <= (others => '0');
		en_TCP <= '1';
      TCP_ra <= (others => '0');
	   TX_fifo_we_i <= '0';
	 else
	   ps_tgl_sync <= ps_tgl_sync(0) & ps_tgl;
	   TCP_wa_sync <= TCP_wa_sync(2) & TCP_wa_gray;
		if(TCP_wa_sync(3 downto 2) = "10")then
		  TCP_wap(7 downto 3) <= TCP_wap(7 downto 3) + 1;
		end if;
		TCP_wap(2) <= TCP_wa_sync(2);
		TCP_wap(1) <= TCP_wa_sync(2) xor TCP_wa_sync(1);
		TCP_wap(0) <= TCP_wa_sync(2) xor TCP_wa_sync(1) xor TCP_wa_sync(0);
		en_TCP <= not TCP_CLOSE_REQ and (en_TCP or not TCP_ESTABLISH);
	   if(TCP_ESTABLISH = '1' and (TCP_ra(0) = '1' or (en_TCP = '1' and 
	        TX_fifo_afull ='0' and TCP_wap /= TCP_ra(8 downto 1))))then
	     TCP_ra(0) <= not TCP_ra(0);
	     TX_fifo_we_i <= '1';
	   else	
	     TX_fifo_we_i <= '0';
		end if;
      if(TCP_ra(0) = '1')then
		  TCP_ra(8 downto 1) <= TCP_ra(8 downto 1) + 1;
		end if;
	 end if;
  end if;
end process;
process(mclk90,reset)
begin
  if(reset = '1')then
    rd_rqst <= '0';
	 TCP_wr <= '0';
    TCP_wa <= (others => '0');
    TCP_wa_gray <= (others => '0');
	 TCP_ra_sync <= (others => '0');
	 TCP_wa6q <= '0';
    TCP_wc <= (others => '0');
  elsif(mclk90'event and mclk90 = '1')then
    if(mem_rdyp = '0')then
	   rd_rqst <= '0';
    elsif(TCP_wc(1) = '0')then
	   rd_rqst <= '1';
    elsif(TCP_wc = "11")then
	   rd_rqst <= '0';
	 end if;
	 TCP_wr <= rd_burstp;
    if(TCP_wr = '1' and mem_rdyp = '1')then
	   TCP_wa(8 downto 0) <= TCP_wa(8 downto 0) + 1;
	 end if;
	 TCP_wa_gray(0) <= TCP_wa(2) xor TCP_wa(1);
	 TCP_wa_gray(1) <= TCP_wa(3) xor TCP_wa(2);
	 TCP_wa_gray(2) <= TCP_wa(3);
	 TCP_ra_sync <= TCP_ra_sync(0) & TCP_ra(6);
	 TCP_wa6q <= TCP_wa(6);
	 if((TCP_ra_sync = "10") /= (TCP_wa(6) = '0' and TCP_wa6q = '1'))then
	   if(TCP_ra_sync = "10")then
		  TCP_wc <= TCP_wc - 1;
		else
		  TCP_wc <= TCP_wc + 1;
		end if;
	 end if;
  end if;
end process;
process(mclk180,reset)
begin
  if(reset = '1')then
    TKO_ra <= (others => '0');
	 mem_rdyp <= '0';
  elsif(mclk180'event and mclk180 = '1')then
    TKO_ra(0) <= w_cycle and not casn;
    if(TKO_ra(0) = '1')then
	   TKO_ra(8 downto 1) <= TKO_ra(8 downto 1) + 1;
	 end if;
	 mem_rdyp <= mem_rdy;
  end if;
end process;
process(mclk0,reset)
begin
  if(reset = '1')then
    fifo_wr <= '0';
	 TKO_wa_sync <= (others => '0');
    TKO_wc <= (others => '0');
  elsif(mclk0'event and mclk0 = '1')then
    fifo_wr <= w_cycle and not casn;
	 TKO_wa_sync(1 downto 0) <= TKO_wa_sync(0) & TKO_wa(0);
	 TKO_wa_sync(2) <= TKO_wa_sync(1) xor TKO_wa_sync(0);
	 if(TKO_wa_sync(2) /= fifo_wr)then
	   if(TKO_wa_sync(2) = '1')then
		  TKO_wc <= TKO_wc + 1;
		else
		  TKO_wc <= TKO_wc - 1;
		end if;
	 end if;
  end if;
end process;
process(mclk0,reset)
begin
  if(reset = '1')then
	 rd_burst <= '0';
    fifo_wa <= (others => '0');
    fifo_ra <= (others => '0');
    fifo_wc <= (others => '0');
	 fifo_afull_i <= '0';
  elsif(mclk0'event and mclk0 = '1')then
	 if(rasn = '0')then
	   rd_burst <= '0';
	 elsif(r_cycle = '1' and casn = '0')then
	   rd_burst <= '1';
	 end if;
	 if(rasn = '1' and casn = '0')then
	   if(wen = '0')then
		  fifo_wa <= fifo_wa + 1;
        fifo_wc <= fifo_wc + 1;
	   elsif(mem_rdy = '1')then
		  fifo_ra <= fifo_ra + 1;
        fifo_wc <= fifo_wc - 1;
	   end if;
	 end if;
    if(fifo_wc(N-1 downto N-4) = x"0")then
	   fifo_afull_i <= '0';
	 elsif(fifo_wc(N-1 downto N-4) = x"f")then
	   fifo_afull_i <= '1';
    end if;
--    if(fifo_wc(N-2 downto N-5) = x"0")then
--	   fifo_afull_i <= '0';
--	 elsif(fifo_wc(N-2 downto N-5) = x"f")then
--	   fifo_afull_i <= '1';
--    end if;
  end if;
end process;
-- ram access request logic
process(mclk0,reset)
begin
  if(reset = '1')then
    rqst <= '0';
    is_read <= '0';
  elsif(mclk0'event and mclk0 = '1')then
    if(precharge = '1' or stop_r = '1')then
      rqst <= '0';
    elsif(TKO_wc /= x"00" or (rd_rqst = '1' and fifo_wc /= zero) or 
			(init_done(1) = '1' and mem_rdy = '0' and ref_rqst = '1'))then
      rqst <= '1';
    end if;
    if(rqst = '0')then
	   if((TKO_wc(7) = '0' and rd_rqst = '1' and fifo_wc /= zero) or (init_done(1) = '1' and mem_rdy = '0'))then
        is_read <= '1';
		else
        is_read <= '0';
		end if;
    end if;
  end if; 
end process;
-- memory address
process(clk)
begin
  if(clk'event and clk = '1')then
    TCP_CLOSE_ACK <= TCP_CLOSE_REQ;
  end if;
end process;
process(mclk0)
begin
  if(mclk0'event and mclk0 = '1')then
-- this ensures active, read/write and precharge are to the same bank
    if(inh_ras = '0')then
      if(is_read = '0')then
        ma(14) <= fifo_wa(N-1);
        ma(13) <= fifo_wa(N-2) or en_dll;
		else
		  ma(14 downto 13) <= fifo_ra(N-1 downto N-2);
--        ma(14) <= fifo_wa(N-14);
--        ma(13) <= fifo_wa(N-15) or en_dll;
--		else
--		  ma(14) <= fifo_ra(N-14);
--		  ma(13) <= fifo_ra(N-15);
--        ma(14) <= fifo_wa(N-1);
--        ma(13) <= fifo_wa(N-15) or en_dll;
--		else
--		  ma(14) <= fifo_ra(N-1);
--		  ma(13) <= fifo_ra(N-15);
	   end if;
	 end if;
	 if(inh_ras = '0')then
      if(is_read = '0')then
        ma(12 downto 11) <= fifo_wa(N-3 downto N-4);
        ma(10) <= fifo_wa(N-5) or (precharge and not init_done(1));
        ma(9) <= fifo_wa(N-6);
        ma(8) <= fifo_wa(N-7) or rst_dll;
        ma(7 downto 6) <= fifo_wa(N-8 downto N-9);
-- latency = 2
        ma(5) <= fifo_wa(N-10) or rst_dll;
        ma(4 downto 2) <= fifo_wa(N-11 downto N-13);
-- burst = 4
        ma(1) <= fifo_wa(N-14) or rst_dll;
        ma(0) <= fifo_wa(N-15);
		else  
	     ma(12 downto 0) <= fifo_ra(N-3 downto N-15);
--        ma(1) <= fifo_wa(N-1) or rst_dll;
--        ma(0) <= fifo_wa(N-2);
--		else  
--	     ma(12 downto 2) <= fifo_ra(N-3 downto N-13);
--	     ma(1) <= fifo_ra(N-1);
--	     ma(0) <= fifo_ra(N-2);
	   end if;
	 else
      if(is_read = '0')then
--        ma(12 downto 0) <= '0' & fifo_wa(8) & '0' & fifo_wa(7 downto 0) & "00"; -- 64MX4
        ma(12 downto 0) <= '0' & fifo_wa(8) & precharge & fifo_wa(7 downto 0) & "00"; -- 64MX4
--	     ma(12 downto 0) <= "0000" & fifo_wa(6 downto 0) & "00"; -- 16MX16
		else  
--        ma(12 downto 0) <= '0' & fifo_ra(8) & '0' & fifo_ra(7 downto 0) & "00"; -- 64MX4
        ma(12 downto 0) <= '0' & fifo_ra(8) & stop_r & fifo_ra(7 downto 0) & "00"; -- 64MX4
--	     ma(12 downto 0) <= "0000" & fifo_ra(6 downto 0) & "00"; -- 16MX16
	   end if;
	 end if;
  end if;
end process;
-- generate read and write burst signals -------------------------------------------------------
process(mclk0,reset)
begin
  if(reset = '1')then
    t_md <= '1';
  elsif(mclk0'event and mclk0 = '1')then
    t_md <= not(not casn and w_cycle) and (t_md or pc_dl);
  end if;
end process;
process(mclk270,reset)
begin
  if(reset = '1')then
    t_dqs <= '1';
	 dqs_out <= '0';
  elsif(mclk270'event and mclk270 = '1')then
    t_dqs <= not(not casn and w_cycle) and (t_dqs or pc_dl);
	 dqs_out <= not t_dqs;
  end if;
end process;
-- sdram state machine -----------------------------------------------------------------------------------------
-- sdram power on initialization states: precharge all banks, enable DLL, reset DLL, precharge all banks,
-- two refresh cycle and set mode register.
i_en_dll: delayN
  generic map(N=> 4) port map(d => start_init, ce => '1', c => mclk0, q => en_dll);
i_rst_dll: delayN
  generic map(N=> 6) port map(d => start_init, ce => '1', c => mclk0, q => rst_dll);
process(mclk0,lock)
variable state: std_logic_vector(2 downto 0);
begin
state := rasn & casn & wen;
  if(lock = '0')then
	 stop_w <= '0';
	 stop_r <= '0';
	 stop_w_q <= '0';
    w_cycle <= '0';
    r_cycle <= '0';
    pc_dl <= '0';
    precharge <= '0';
    prechage_dl <= '0';
    rfc_cntr <= "0000";
	 inh_ras <= '0';
    rasn <= '1';
    casn <= '1';
    wen <= '1';
    init_done <= "00";
    ref_rqst <= '0';
  elsif(mclk0'event and mclk0 = '1')then
	 if((ma(11) = '1' and ma(9 downto 2) = x"ff") or TKO_wc(7 downto 1) = "0000000")then
--	 if(ma(8 downto 2) = "1111111" or TKO_wc(7 downto 1) = "0000000")then
	   stop_w <= w_cycle and not casn;
	 else
	   stop_w <= '0';
	 end if;
	 if((ma(11) = '1' and ma(9 downto 2) = x"ff") or TCP_wc = "11" or
--	 if(ma(8 downto 2) = "1111111" or TCP_wc = "11" or 
	    fifo_wc(N-1 downto 1) = zero(N-1 downto 1))then
	   stop_r <= r_cycle and not casn;
	 else
	   stop_r <= '0';
	 end if;
	 stop_w_q <= stop_w;
	 pc_dl <= stop_w_q or start_init or rst_dll;
    precharge <= pc_dl;
    prechage_dl <= precharge or stop_r;
-- auto refresh tRFC counter
    if(state = "001")then
	   rfc_cntr(3) <= '1';
	 elsif(rfc_cntr(2) = '1')then
	   rfc_cntr(3) <= '0';
	 end if;
	 if(rfc_cntr(3) = '0')then
	   rfc_cntr(2 downto 0) <= "000";
	 else
	   rfc_cntr(2 downto 0) <= rfc_cntr(2 downto 0) + 1;
	 end if;
-- inhibit RAS after refresh or read/write until tRFC or tPR
    inh_ras <= not prechage_dl and not (rfc_cntr(2) and rfc_cntr(0)) and ((not rasn and wen) or inh_ras) ;
-- latch refresh request
    if(ref = '1')then
      ref_rqst <= '1';
    end if;
-- default to go to NOP
    rasn <= '1';
    casn <= '1';
    wen <= '1';
    if(state = "011")then
	   w_cycle <= not is_read;
	 elsif(stop_w = '1')then
	   w_cycle <= '0';
	 end if;
    if(state = "011")then
	   r_cycle <= is_read;
	 elsif(rasn = '0')then
	   r_cycle <= '0';
	 end if;
    case state is
-- NOP
      when "111" =>
---- goto precharge
        if(precharge = '1' or stop_r = '1')then
          rasn <= '0';
          wen <= '0';
        end if;
---- goto auto refresh
        if(ref_rqst = '1' and inh_ras = '0')then
          rasn <= '0';
          casn <= '0';
        end if;
---- goto set mode
        if(en_dll = '1' or rst_dll = '1')then
          rasn <= '0';
          casn <= '0';
          wen <= '0';
        end if;
---- goto active
        if(rqst = '1' and inh_ras = '0')then
          rasn <= '0';
        end if;
---- goto write
        if(w_cycle = '1' and stop_w = '0')then
          casn <= '0';
          wen <= '0';
        end if;
---- goto read
        if(r_cycle = '1' and stop_r = '0')then
          casn <= '0';
          wen <= '1';
        end if;
-- AUTO REFRESH
      when "001" =>
		  ref_rqst <= '0';
        init_done <= init_done(0) & '1';
      when others =>
      null;
    end case;
  end if;
end process;
----------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------
-- instantiate refresh generator
I_refresh: refresh PORT MAP(
		clk => mclk0,
		lock => lock,
		init_done => init_done(1),
		start_init => start_init,
		ddr_csn => cs_n,
		ddr_clke => mclke,
		ref => ref
	);
end sdram_fifo_arch;
----------------------------------------------------------------
library IEEE,mylib;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use mylib.atm_pack.ALL;

entity refresh is
    Port ( clk : in std_logic;
           lock : in std_logic;
           init_done : in std_logic;
           start_init : out std_logic;
           ddr_csn : out std_logic;
           ddr_clke : out std_logic;
           ref : out std_logic);
end refresh;

architecture refresh_arch of refresh is
signal clke,tc: std_logic;
signal startup: std_logic_vector(4 downto 0);
signal div: std_logic_vector(9 downto 0);
begin
ddr_clke <= clke;
-- refresh signal generator
process(clk,lock)
begin
  if(lock = '0')then
    tc <= '0';
    ref <= '0';
    clke <= '0';
    start_init <= '0';
    ddr_csn <= '1';
	 startup <= (others => '0');
	 div <= (others => '0');
  elsif(clk'event and clk = '1')then
    ddr_csn <= '0';
    if(startup = "11011" and tc = '1')then
      start_init <= '1';
      clke <= '1';
    else
      start_init <= '0';
    end if;
    ref <= tc and clke;
	 if(init_done = '1')then
	   startup <= (others => '1');
	 elsif(tc = '1')then
	   startup <= startup + 1;
	 end if;
	 if(div(9 downto 8) = "11")then
	   div <= (others => '0');
	   tc <= '1';
	 else
	   div <= div + 1;
	   tc <= '0';
	 end if;
  end if;
end process;
end refresh_arch;
-----------------------------------------------------------------------------------
-- delay d from 2 to 17 clocks with ce = '1'
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
library UNISIM;
use UNISIM.VComponents.all;

entity delayN is
    generic (N: positive := 2;
	          INIT : bit_vector := x"0000");
    port (
        c: in STD_LOGIC;
        d: in STD_LOGIC;
        ce: in STD_LOGIC;
        q: out STD_LOGIC
    );
end delayN;
architecture delayN_arch of delayN is
signal a: std_logic_vector(4 downto 0);
signal dl: std_logic;
begin
a <= conv_std_logic_vector(N-2,5);
delay: srl16e
    generic map(INIT => INIT)
    port map(
           d => d,
           ce => ce,
           clk => c,
           a0 => a(0),
           a1 => a(1),
           a2 => a(2),
           a3 => a(3),
           q => dl);
process(c)
begin
  if(c'event and c = '1')then
    if(ce = '1')then
      q <= dl;
    end if;
  end if;
end process;
end delayN_arch;
----------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
library UNISIM;
use UNISIM.VComponents.all;

entity ddr_io is
    port (
        clk: in STD_LOGIC;
        clkn: in STD_LOGIC;
        pad: inout std_logic;
        t: in STD_LOGIC;
        d0: in STD_LOGIC;
        d1: in STD_LOGIC;
        iclk: in STD_LOGIC;
        iclkn: in STD_LOGIC;
        q0: out std_logic; -- rising edge       
        q1: out std_logic  -- falling edge    
    );
end ddr_io;

architecture ddr_io_arch of ddr_io is
signal din, oq, tq: std_logic;
attribute IOB : string;
attribute IOB of q0_reg,q1_reg,tq_reg: label is "TRUE";
begin
ib: ibuf port map(i => pad, o => din);
q0_reg: fd port map(d => din, c=> iclk, q => q0);
q1_reg: fd port map(d => din, c=> iclkn, q => q1);
oq_reg:  fddrrse
    port map(
          d0 => d0,
          d1 => d1,
          c0 => clk,
          c1 => clkn,
          ce => '1',
          r  => '0',
          s  => '0',
          q  => oq);
tq_reg:  fd
    port map(
          d => t,
          c => clk,
          q  => tq);
ob: obuft port map( i => oq, o => pad, t => tq);
end ddr_io_arch;
